# The LearningOnline Network
# Cookie Based Access Handler
#
# $Id: lonacc.pm,v 1.88 2006/07/21 16:07:48 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###

package Apache::lonacc;

use strict;
use Apache::Constants qw(:common :http :methods);
use Apache::File;
use Apache::lonnet;
use Apache::loncommon();
use Apache::lonlocal;
use Apache::restrictedaccess();
use CGI::Cookie();
use Fcntl qw(:flock);
use LONCAPA;

sub cleanup {
    my ($r)=@_;
    if (! $r->is_initial_req()) { return DECLINED; }
    &Apache::lonnet::save_cache();
    return OK;
}

sub goodbye {
    my ($r)=@_;
    &Apache::lonnet::goodbye();
    return DONE;
}

###############################################

sub get_posted_cgi {
    my ($r) = @_;

    my $buffer;
    if ($r->header_in('Content-length')) {
	$r->read($buffer,$r->header_in('Content-length'),0);
    }
    unless ($buffer=~/^(\-+\w+)\s+Content\-Disposition\:\s*form\-data/si) {
	my @pairs=split(/&/,$buffer);
	my $pair;
	foreach $pair (@pairs) {
	    my ($name,$value) = split(/=/,$pair);
	    $value =~ tr/+/ /;
	    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
	    $name  =~ tr/+/ /;
	    $name  =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
	    &Apache::loncommon::add_to_env("form.$name",$value);
	}
    } else {
	my $contentsep=$1;
	my @lines = split (/\n/,$buffer);
	my $name='';
	my $value='';
	my $fname='';
	my $fmime='';
	my $i;
	for ($i=0;$i<=$#lines;$i++) {
	    if ($lines[$i]=~/^$contentsep/) {
		if ($name) {
		    chomp($value);
		    if ($fname) {
			$env{"form.$name.filename"}=$fname;
			$env{"form.$name.mimetype"}=$fmime;
		    } else {
			$value=~s/\s+$//s;
		    }
		    &Apache::loncommon::add_to_env("form.$name",$value);
		}
		if ($i<$#lines) {
		    $i++;
		    $lines[$i]=~
		/Content\-Disposition\:\s*form\-data\;\s*name\=\"([^\"]+)\"/i;
		    $name=$1;
		    $value='';
		    if ($lines[$i]=~/filename\=\"([^\"]+)\"/i) {
			$fname=$1;
			if 
                            ($lines[$i+1]=~/Content\-Type\:\s*([\w\-\/]+)/i) {
				$fmime=$1;
				$i++;
			    } else {
				$fmime='';
			    }
		    } else {
			$fname='';
			$fmime='';
		    }
		    $i++;
		}
	    } else {
		$value.=$lines[$i]."\n";
	    }
	}
    }
#
# Digested POSTed values
#
# Remember the way this was originally done (GET or POST)
#
    $env{'request.method'}=$ENV{'REQUEST_METHOD'};
#
# There may also be stuff in the query string
# Tell subsequent handlers that this was GET, not POST, so they can access query string.
# Also, unset POSTed content length to cover all tracks.
#

    $r->method_number(M_GET);

    $r->method('GET');
    $r->headers_in->unset('Content-length');
}

sub portfolio_access {
    my ($r,$requrl) = @_;
    my $access=&Apache::lonnet::allowed('bre',$requrl);
    if ($access eq '2' || $access eq 'F') {
	return OK;
    }
    my (undef,$udom,$unum,$file_name,$group) = &parse_portfolio_url($requrl);
    my $result = &get_portfolio_access($udom,$unum,$file_name,$group);
    &Apache::lonnet::logthis("got pa of $result");
    if ($result eq 'ok') {
	return OK;
    } elsif ($result =~ /^[^:]+:guest_/) {
	&Apache::lonnet::logthis("doign pac $result");
	&passphrase_access_checker($r,$result,$requrl);
	return OK;
    }
    return undef;
}

sub get_portfolio_access {
    my ($udom,$unum,$file_name,$group) = @_;
 
    my $current_perms = &Apache::lonnet::get_portfile_permissions($udom,$unum);
    my %access_controls = &Apache::lonnet::get_access_controls(
                                             $current_perms,$group,$file_name);
    my ($public,$guest,@domains,@users,@courses,@groups);
    my $now = time;
    my $access_hash = $access_controls{$file_name};
    if (ref($access_hash) eq 'HASH') {
        foreach my $key (keys(%{$access_hash})) {
            my ($num,$scope,$end,$start) = ($key =~ /^([^:]+):([a-z]+)_(\d*)_?(\d*)$/);
            if ($start > $now) {
                next;
            }
            if ($end && $end<$now) {
                next;
            }
            if ($scope eq 'public') {
                $public = $key;
                last;
            } elsif ($scope eq 'guest') {
                $guest = $key;
            } elsif ($scope eq 'domains') {
                push(@domains,$key);
            } elsif ($scope eq 'users') {
                push(@users,$key);
            } elsif ($scope eq 'course') {
                push(@courses,$key);
            } elsif ($scope eq 'group') {
                push(@groups,$key);
            }
        }
        if ($public) {
            return 'ok';
        }
        if ($env{'user.name'} eq 'public' && $env{'user.domain'} eq 'public') {
            if ($guest) {
                return $guest;
            }
        } else {
            if (@domains > 0) {
                foreach my $domkey (@domains) {
                    if (ref($access_hash->{$domkey}{'dom'}) eq 'ARRAY') {
                        if (grep(/^\Q$env{'user.domain'}\E$/,@{$access_hash->{$domkey}{'dom'}})) {
                            return 'ok';
                        }
                    }
                }
            }
            if (@users > 0) {
                foreach my $userkey (@users) {
                    if (exists($access_hash->{$userkey}{'users'}{$env{'user.name'}.':'.$env{'user.domain'}})) {
                        return 'ok';
                    }
                }
            }
            my %roleshash;
            my @courses_and_groups = @courses;
            push(@courses_and_groups,@groups); 
            if (@courses_and_groups > 0) {
                my (%allgroups,%allroles); 
                my ($start,$end,$role,$sec,$group);
                foreach my $envkey (%env) {
                    if ($envkey =~ m-^user\.role\.(gr|cc|in|ta|ep|st)\./([^/]+)/([^/]+)/?([^/]*)$-) {
                        my $cid = $2.'_'.$3; 
                        if ($1 eq 'gr') {
                            $group = $4;
                            $allgroups{$cid}{$group} = $env{$envkey};
                        } else {
                            if ($4 eq '') {
                                $sec = 'none';
                            } else {
                                $sec = $4;
                            }
                            $allroles{$cid}{$1}{$sec} = $env{$envkey};
                        }
                    } elsif ($envkey =~ m-^user\.role\./cr/(\w+/\w+/\w*)./([^/]+)/([^/]+)/?([^/]*)$-) {
                        my $cid = $2.'_'.$3;
                        if ($4 eq '') {
                            $sec = 'none';
                        } else {
                            $sec = $4;
                        }
                        $allroles{$cid}{$1}{$sec} = $env{$envkey};
                    }
                }
                if (keys(%allroles) == 0) {
                    return;
                }
                foreach my $key (@courses_and_groups) {
                    my %content = %{$$access_hash{$key}};
                    my $cnum = $content{'number'};
                    my $cdom = $content{'domain'};
                    my $cid = $cdom.'_'.$cnum;
                    if (!exists($allroles{$cid})) {
                        next;
                    }    
                    foreach my $role_id (keys(%{$content{'roles'}})) {
                        my @sections = @{$content{'roles'}{$role_id}{'section'}};
                        my @groups = @{$content{'roles'}{$role_id}{'group'}};
                        my @status = @{$content{'roles'}{$role_id}{'access'}};
                        my @roles = @{$content{'roles'}{$role_id}{'role'}};
                        foreach my $role (keys(%{$allroles{$cid}})) {
                            if ((grep/^all$/,@roles) || (grep/^\Q$role\E$/,@roles)) {
                                foreach my $sec (keys(%{$allroles{$cid}{$role}})) {
                                    if (&course_group_datechecker($allroles{$cid}{$role}{$sec},$now,\@status) eq 'ok') {
                                        if (grep/^all$/,@sections) {
                                            return 'ok';
                                        } else {
                                            if (grep/^$sec$/,@sections) {
                                                return 'ok';
                                            }
                                        }
                                    }
                                }
                                if (keys(%{$allgroups{$cid}}) == 0) {
                                    if (grep/^none$/,@groups) {
                                        return 'ok';
                                    }
                                } else {
                                    if (grep/^all$/,@groups) {
                                        return 'ok';
                                    } 
                                    foreach my $group (keys(%{$allgroups{$cid}})) {
                                        if (grep/^$group$/,@groups) {
                                            return 'ok';
                                        }
                                    }
                                } 
                            }
                        }
                    }
                }
            }
            if ($guest) {
                return $guest;
            }
        }
    }
    return;
}

sub passphrase_access_checker {
    my ($r,$guestkey,$requrl) = @_;
    my ($num,$scope,$end,$start) = ($guestkey =~ /^([^:]+):([a-z]+)_(\d*)_?(\d*)$/);
    if ($scope eq 'guest') {
        if (exists($env{'user.passphrase_access_'.$requrl})) {
            if (($env{'user.passphrase_access_'.$requrl} == 0) || 
                ($env{'user.passphrase_access_'.$requrl} > time)) {
                $env{'request.publicaccess'} = 1;
                return 'ok'; 
            }
        }
    }
    $r->set_handlers('PerlHandler'=> \&Apache::restrictedaccess::handler);
    $r->content_type('perl-script');
    return;
}

sub course_group_datechecker {
    my ($dates,$now,$status) = @_;
    my ($start,$end) = split(/\./,$dates);
    if (!$start && !$end) {
        return 'ok';
    }
    if (grep/^active$/,@{$status}) {
        if (((!$start) || ($start && $start <= $now)) && ((!$end) || ($end && $end >= $now))) {
            return 'ok';
        }
    }
    if (grep/^previous$/,@{$status}) {
        if ($end > $now ) {
            return 'ok';
        }
    }
    if (grep/^future$/,@{$status}) {
        if ($start > $now) {
            return 'ok';
        }
    }
    return; 
}

sub parse_portfolio_url {
    my ($url) = @_;

    my ($type,$udom,$unum,$group,$file_name);
    
    if ($url =~  m-/+uploaded/([^/]+)/([^/]+)/portfolio(/.+)$-) {
	$type = 1;
        $udom = $1;
        $unum = $2;
        $file_name = $3;
    } elsif ($url =~ m-/+uploaded/([^/]+)/([^/]+)/groups/([^/]+)/portfolio/(.+)$-) {
	$type = 2;
        $udom = $1;
        $unum = $2;
        $group = $3;
        $file_name = $3.'/'.$4;
    }
    if (wantarray) {
	return ($type,$udom,$unum,$file_name,$group);
    }
    return $type;
}

sub is_portfolio_url {
    my ($url) = @_;
    return scalar(&parse_portfolio_url($url));
}

sub handler {
    my $r = shift;
    my $requrl=$r->uri;
    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
    my $lonid=$cookies{'lonID'};
    my $cookie;
    my $lonidsdir=$r->dir_config('lonIDsDir');

    my $handle;
    if ($lonid) {
	$handle=$lonid->value;
        $handle=~s/\W//g;
    }
      
    my ($sso_login);
    if ($r->user 
	&& (!$lonid || !-e "$lonidsdir/$handle.id" || $handle eq '') ) {
	$sso_login = 1;
	my $domain = $r->dir_config('lonDefDomain');
	my $home=&Apache::lonnet::homeserver($r->user,$domain);
	if ($home !~ /(con_lost|no_such_host)/) {
	    $handle=&Apache::lonauth::success($r,$r->user,$domain,
					     $home,'noredirect');
	    $r->header_out('Set-cookie',"lonID=$handle; path=/");
	}
    }

    if ($sso_login) {
	&Apache::lonnet::appenv('request.sso.login' => 1);
    }

    if ($r->dir_config("lonBalancer") eq 'yes') {
	$r->set_handlers('PerlResponseHandler'=>
			 [\&Apache::switchserver::handler]);
    }

    if ($handle ne '') {
        if ((-e "$lonidsdir/$handle.id") && ($handle ne '')) {

# ------------------------------------------------------ Initialize Environment

            &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);

# --------------------------------------------------------- Initialize Language

	    &Apache::lonlocal::get_language_handle($r);

# -------------------------------------------------------------- Resource State

            if ($requrl=~/^\/+(res|uploaded)\//) {
               $env{'request.state'} = "published";
	    } else {
	       $env{'request.state'} = 'unknown';
            }
            $env{'request.filename'} = $r->filename;
            $env{'request.noversionuri'} = &Apache::lonnet::deversion($requrl);
# -------------------------------------------------------- Load POST parameters

	    &Apache::lonacc::get_posted_cgi($r);

# ---------------------------------------------------------------- Check access
            my $now = time;
	    if (&is_portfolio_url($requrl)) {
		my $result = &portfolio_access($r,$requrl);
		if (defined($result)) { return $result; }
	    }
            if ($requrl!~/^\/adm|public|prtspool\//) {
		my $access=&Apache::lonnet::allowed('bre',$requrl);
                if ($access eq '1') {
		   $env{'user.error.msg'}="$requrl:bre:0:0:Choose Course";
	           return HTTP_NOT_ACCEPTABLE; 
                }
                if (($access ne '2') && ($access ne 'F')) {
		   $env{'user.error.msg'}="$requrl:bre:1:1:Access Denied";
	           return HTTP_NOT_ACCEPTABLE; 
                }
            }
	    if ($requrl =~ m|^/prtspool/|) {
		my $start='/prtspool/'.$env{'user.name'}.'_'.
		    $env{'user.domain'};
		if ($requrl !~ /^\Q$start\E/) {
		    $env{'user.error.msg'}="$requrl:bre:1:1:Access Denied";
		    return HTTP_NOT_ACCEPTABLE;
		}
	    }
	    if ($env{'user.name'} eq 'public' && 
		$env{'user.domain'} eq 'public' &&
		$requrl !~ m{^/+(res|public)/} &&
		$requrl !~ m{^/+adm/(help|logout|randomlabel\.png)}) {
		$env{'request.querystring'}=$r->args;
		$env{'request.firsturl'}=$requrl;
		return FORBIDDEN;
	    }
# ------------------------------------------------------------- This is allowed
          if ($env{'request.course.id'}) {
	    &Apache::lonnet::countacc($requrl);
            $requrl=~/\.(\w+)$/;
            if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
 ($requrl=~/^\/adm\/.*\/(aboutme|navmaps|smppg|bulletinboard)(\?|$)/) ||
 ($requrl=~/^\/adm\/wrapper\//) ||
 ($requrl=~m|^/adm/coursedocs/showdoc/|) ||
 ($requrl=~m|\.problem/smpedit$|) ||
 ($requrl=~/^\/public\/.*\/syllabus$/)) {
# ------------------------------------- This is serious stuff, get symb and log
		my $query=$r->args;
                my $symb;
                if ($query) {
		    &Apache::loncommon::get_unprocessed_cgi($query,['symb']);
                }
                if ($env{'form.symb'}) {
		    $symb=&Apache::lonnet::symbclean($env{'form.symb'});
                    if ($requrl =~ m|^/adm/wrapper/|
			|| $requrl =~ m|^/adm/coursedocs/showdoc/|) {
                        my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
                        &Apache::lonnet::symblist($map,$murl => [$murl,$mid],
						  'last_known' =>[$murl,$mid]);
                    } elsif ((&Apache::lonnet::symbverify($symb,$requrl)) ||
			     (($requrl=~m|(.*)/smpedit$|) &&
			      &Apache::lonnet::symbverify($symb,$1))) {
                      my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
		      &Apache::lonnet::symblist($map,$murl => [$murl,$mid],
						'last_known' =>[$murl,$mid]);
		    } else {
			$r->log_reason('Invalid symb for '.$requrl.': '.
                                       $symb);
		        $env{'user.error.msg'}=
                                "$requrl:bre:1:1:Invalid Access";
  	                return HTTP_NOT_ACCEPTABLE; 
                    }
                } else {
	            $symb=&Apache::lonnet::symbread($requrl);
		    if (&Apache::lonnet::is_on_map($requrl) && $symb &&
			!&Apache::lonnet::symbverify($symb,$requrl)) {
			$r->log_reason('Invalid symb for '.$requrl.': '.$symb);
		        $env{'user.error.msg'}=
                                "$requrl:bre:1:1:Invalid Access";
  	                return HTTP_NOT_ACCEPTABLE; 
		    }
		    if ($symb) {
			my ($map,$mid,$murl)=
			    &Apache::lonnet::decode_symb($symb);
			&Apache::lonnet::symblist($map,$murl =>[$murl,$mid],
						'last_known' =>[$murl,$mid]);
		    }
                }
                $env{'request.symb'}=$symb;
                &Apache::lonnet::courseacclog($symb);
            } else {
# ------------------------------------------------------- This is other content
                &Apache::lonnet::courseacclog($requrl);    
            }
	  }
            return OK; 
        } else { 
            $r->log_reason("Cookie $handle not valid", $r->filename); 
        }
	}

# -------------------------------------------- See if this is a public resource
    if ($requrl=~m|^/public/|
	|| (&Apache::lonnet::metadata($requrl,'copyright') eq 'public')) {
        &Apache::lonnet::logthis('Granting public access: '.$requrl);
        &Apache::lonlocal::get_language_handle($r);
	my $cookie=
	    &Apache::lonauth::success($r,'public','public','public');
        my $lonidsdir=$r->dir_config('lonIDsDir');
	&Apache::lonnet::transfer_profile_to_env($lonidsdir,$cookie);
	&Apache::lonacc::get_posted_cgi($r);
        $env{'request.state'} = "published";
        $env{'request.publicaccess'} = 1;
        $env{'request.filename'} = $r->filename;

	$r->header_out('Set-cookie',"lonID=$cookie; path=/");
        return OK;
    }
    if ($requrl=~m|^/+adm/+help/+|) {
	return OK;
    }
# ------------------------------------- See if this is a viewable portfolio file
    if (&is_portfolio_url($requrl)) {
	my $result = &portfolio_access($r,$requrl);
	if (defined($result)) { return $result; }
    }

# -------------------------------------------------------------- Not authorized
    $requrl=~/\.(\w+)$/;
#    if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
#        ($requrl=~/^\/adm\/(roles|logout|email|menu|remote)/) ||
#        ($requrl=~m|^/prtspool/|)) {
# -------------------------- Store where they wanted to go and get login screen
	$env{'request.querystring'}=$r->args;
	$env{'request.firsturl'}=$requrl;
       return FORBIDDEN;
#   } else {
# --------------------------------------------------------------------- Goodbye
#       return HTTP_BAD_REQUEST;
#   }
}

1;
__END__

=head1 NAME

Apache::lonacc - Cookie Based Access Handler

=head1 SYNOPSIS

Invoked (for various locations) by /etc/httpd/conf/srm.conf:

 PerlAccessHandler       Apache::lonacc

=head1 INTRODUCTION

This module enables cookie based authentication and is used
to control access for many different LON-CAPA URIs.

Whenever the client sends the cookie back to the server, 
this cookie is handled by either lonacc.pm or loncacc.pm
(see srm.conf for what is invoked when).  If
the cookie is missing or invalid, the user is re-challenged
for login information.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

=head1 HANDLER SUBROUTINE

This routine is called by Apache and mod_perl.

=over 4

=item *

transfer profile into environment

=item *

load POST parameters

=item *

check access

=item *

if allowed, get symb, log, generate course statistics if applicable

=item *

otherwise return error

=item *

see if public resource

=item *

store attempted access

=back

=cut
