# The LearningOnline Network with CAPA 
# definition of tags that give a structure to a document
#
# $Id: bridgetask.pm,v 1.181 2006/07/17 20:35:11 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###


package Apache::bridgetask; 

use strict;
use Apache::lonnet;
use Apache::File();
use Apache::lonmenu;
use Apache::lonlocal;
use Apache::lonxml;
use Apache::slotrequest();
use Time::HiRes qw( gettimeofday tv_interval );
use lib '/home/httpd/lib/perl/';
use LONCAPA;
 

BEGIN {
    &Apache::lonxml::register('Apache::bridgetask',('Task','IntroParagraph','Dimension','Question','QuestionText','Setup','Instance','InstanceText','Criteria','ClosingParagraph'));
}

my %dimension;
sub initialize_bridgetask {
    # id of current Dimension, 0 means that no dimension is current 
    # (inside <Task> only)
    @Apache::bridgetask::dimension=();
    # list of all Dimension ids seen
    %Apache::bridgetask::top_dimensionlist=();
    # list of all current Instance ids
    %Apache::bridgetask::instance=();
    # list of all Instance ids seen in this problem
    @Apache::bridgetask::instancelist=();
    # key of queud user data that we are currently grading
    $Apache::bridgetask::queue_key='';
    undef(%dimension);
}

sub proctor_check_auth {
    my ($slot_name,$slot,$type)=@_;
    my $user=$env{'form.proctorname'};
    my $domain=$env{'form.proctordomain'};
    
    my @allowed=split(",",$slot->{'proctor'});
    foreach my $possible (@allowed) {
	my ($puser,$pdom)=(split(':',$possible));
	if ($puser eq $user && $pdom eq $domain) {
	    my $authenticated=0;
	    if ( $slot->{'secret'} =~ /\S/ &&
		 $env{'form.proctorpassword'} eq $slot->{'secret'} ) {
		$authenticated=1;
	    } else {
		
		my $authhost=&Apache::lonnet::authenticate($puser,$env{'form.proctorpassword'},$pdom);
		if ($authhost ne 'no_host') {
		    $authenticated=1;
		}
	    }
	    if ($authenticated) {
		&check_in($type,$user,$domain,$slot_name);
		return 1;
	    }
	}
    }
    return 0;
}

sub check_in {
    my ($type,$user,$domain,$slot_name) = @_;
    my $useslots = &Apache::lonnet::EXT("resource.0.useslots");
    if ( $useslots eq 'map_map') {
	&check_in_sequence($user,$domain,$slot_name);
    } else {
	&create_new_version($type,$user,$domain,$slot_name);
    }
    return 1;
}

sub check_in_sequence {
    my ($user,$domain,$slot_name) = @_;
    my $navmap = Apache::lonnavmaps::navmap->new();
    my ($symb) = &Apache::lonxml::whichuser();
    my ($map)  = &Apache::lonnet::decode_symb($symb);
    my @resources = 
	$navmap->retrieveResources($map, sub { $_[0]->is_problem() },0,0);
    my %old_history = %Apache::lonhomework::history;
    my %old_results = %Apache::lonhomework::results;

    foreach my $res (@resources) {
	&Apache::lonxml::debug("doing ".$res->src);
	&Apache::structuretags::initialize_storage($res->symb);
	my $type = ($res->is_task()) ? 'Task' : 'problem';
	&create_new_version($type,$user,$domain,$slot_name);
	&Apache::structuretags::finalize_storage($res->symb);
    }
    
    %Apache::lonhomework::history = %old_history;
    %Apache::lonhomework::results = %old_results;
}

sub create_new_version {
    my ($type,$user,$domain,$slot_name) = @_;
    
    my $id = '0';
    if ($type eq 'Task') {
	# increment version
	my $version=
	    $Apache::lonhomework::history{'resource.0.version'};
	$version++;
	&Apache::lonxml::debug("Making version $version");
	#clean out all current results
	foreach my $key (keys(%Apache::lonhomework::history)) {
	    if ($key=~/^resource\.0\./) {
		$Apache::lonhomework::results{$key}='';
	    }
	}
	
	#setup new version and who did it
       	$Apache::lonhomework::results{'resource.0.version'}=$version;
	$id = "$version.0";
	if (!defined($user) || !defined($domain)) {
	    $user = $env{'user.name'};
	    $domain = $env{'user.domain'};
	}
	
    } elsif ($type eq 'problem') {
	&Apache::lonxml::debug("authed $slot_name");
    }
    if (!defined($user) || !defined($domain)) {
	$user = $env{'user.name'};
	$domain = $env{'user.domain'};
    }

    $Apache::lonhomework::results{"resource.$id.checkedin"}=
	$user.':'.$domain;

    if (defined($slot_name)) {
	$Apache::lonhomework::results{"resource.$id.checkedin.slot"}=
	    $slot_name;
    }
}

sub get_version {
    my ($version,$previous);
    if ($env{'form.previousversion'} && 
	$env{'form.previousversion'} ne 'current' &&
	defined($Apache::lonhomework::history{'resource.'.$env{'form.previousversion'}.'.0.status'})) {
	$version=$env{'form.previousversion'};
	$previous=1;
    } else {
	if (defined($Apache::lonhomework::results{'resource.0.version'})) {
	    $version=$Apache::lonhomework::results{'resource.0.version'};
	} elsif (defined($Apache::lonhomework::history{'resource.0.version'})) {
	    $version=$Apache::lonhomework::history{'resource.0.version'};
	}
	$previous=0;
    }
    if (wantarray) {
	return ($version,$previous);
    }
    return $version;
}

sub add_previous_version_button {
    my ($status)=@_;
    my $result;
    if ($Apache::lonhomework::history{'resource.0.version'} eq '') {
	return '';
    }
    if ($Apache::lonhomework::history{'resource.0.version'} < 2 &&
	$status ne 'NEEDS_CHECKIN') {
	return '';
    }
    my $version=&get_version();
    if ($env{'form.previousversion'} ne '' &&
	$env{'form.previousversion'} eq $version) {
	$result.="<h3>".&mt("Showing previous version [_1]",$version).
	    "</h3>\n";
    }
    my @to_show;
    foreach my $test_version (1..$Apache::lonhomework::history{'resource.0.version'}) {
	if (defined($Apache::lonhomework::history{'resource.'.$test_version.'.0.status'})) {
	    push(@to_show,$test_version);
	}
    }
    my $list='<option>'.
	join("</option>\n<option>",@to_show).
	     "</option>\n";
    $list.='<option value="current">'.&mt('Current').'</option>';
    $result.='<form name="getprevious" method="post" action="';
    my $uri=$env{'request.uri'};
    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
    $result.=$uri.'">'.
	&mt(' Show a previously done version: [_1]','<select onchange="this.form.submit()" name="previousversion">
<option>'.&mt('Pick one').'</option>
'.$list.'
</select>')."</form>";
    return $result;
}

sub add_grading_button {
    my (undef,$cid)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my %sections = &Apache::loncommon::get_sections($cdom,$cnum);

    my $size=5;
    if (scalar(keys(%sections)) < 3) {
	$size=scalar(keys(%sections))+2;
    }
    my $sec_select = '<select multiple="multiple" name="chosensections" size="'.$size.'">'."\n";
    $sec_select .= "<option value='all' selected='selected'>all</option>\n";
    foreach my $sec (sort {lc($a) cmp lc($b)} (keys(%sections))) {
	$sec_select .= "<option value=\"$sec\">$sec</option>\n";
    }
    $sec_select .= "<option value='none'>none</option></select>\n";
    
    my $result=' <input type="submit" name="gradeasubmission" value="'.
	&mt("Get a submission to grade").'" />';
    $result.='<input type="hidden" name="grade_target" value="webgrade" />';
    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
	my ($entries,$ready,$locks)=&get_queue_counts('gradingqueue');
	$result.='<table><tr>';
	$result.='<td rowspan="4">Specify a section: </td><td rowspan="4">'.$sec_select.'</td>';
	$result.='<td>'.' <input type="submit" name="reviewagrading" value="'.
	    &mt("Select an entry from the grading queue:").'" /> ';

	$result.= &mt("[_1] entries, [_2] ready, [_3] being graded",$entries,$ready,$locks).' </td></tr>'."\n";

	($entries,$ready,$locks)=&get_queue_counts('reviewqueue');
	$result.='<tr><td>'.
	    ' <input type="submit" name="reviewasubmission" value="'.
	    &mt("Select an entry from the review queue:").'" /> ';
	$result.=&mt("[_1] entries, [_2] ready, [_3] being graded",
		     $entries,$ready,$locks).'</td></tr>'."\n";
	$result.='<tr><td> <input type="submit" name="regradeasubmission" value="'.
	    &mt("List of user's grade status").'" /> </td></tr></table>'."\n";
	$result.='<p> <input type="submit" name="regradeaspecificsubmission" value="'.
	    &mt("Regrade specific user:").'" />'."\n";
	$result.='<input type="text" size="12" name="gradinguser" />';
	$result.=&Apache::loncommon::select_dom_form($env{'user.domain'},
						     'gradingdomain');
	$result.=' '.
	    &Apache::loncommon::selectstudent_link('gradesubmission',
						   'gradinguser',
						   'gradingdomain');
	$result.=&Apache::loncommon::studentbrowser_javascript();
	$result.= '</p>';
    }
    return $result;
}

sub add_request_another_attempt_button {
    my ($text)=@_;
    if (!$text) { $text="Request another attempt"; }
    my $result;
    my $symb=&Apache::lonnet::symbread();
    # not a slot access based resource
    my $useslots = &Apache::lonnet::EXT("resource.0.useslots",$symb);
    if ($useslots =~ /^\s*no\s*$/i) {
	return '';
    }

    my ($slot_name,$slot)=&Apache::slotrequest::check_for_reservation($symb);
    my $action='get_reservation';
    if ($slot_name) {
	$text="Change reservation.";
	$action='change_reservation';
	my $description=&Apache::slotrequest::get_description($slot_name,
							      $slot);
	$result.=(<<STUFF);
<p> Will be next available: $description </p>
STUFF
    }
    
    if ($env{'request.enc'}) { $symb=&Apache::lonenc::encrypted($symb); }
    $symb=&escape($symb);
    $result.='<form method="post" action="/adm/slotrequest">'.
	'<input type="hidden" name="symb" value="'.$symb.'" />'.
	'<input type="hidden" name="command" value="'.$action.'" />'.
	'<input type="submit" name="requestattempt" value="'.
	&mt($text).'" />'.
	'</form>';
    return $result;
}

sub preserve_grade_info {
    my $result;
    # if we are viewing someone else preserve that info
    if (defined $env{'form.grade_symb'}) {
	foreach my $field ('symb','courseid','domain','username') {
	    $result .= '<input type="hidden" name="grade_'.$field.
		'" value="'.$env{"form.grade_$field"}.'" />'."\n";
	}
    }
    return $result;
}

sub style {
    my ($target) = @_;
    if ($target eq 'web'
	|| $target eq 'webgrade') {
	return (<<STYLE);
<link rel="stylesheet" type="text/css" href="/res/adm/includes/task.css" />
STYLE
    }
    return;
}

sub show_task {
    my ($status,$previous)=@_;
    if (!$previous && (
		       ( $status eq 'CLOSED' ) ||
		       ( $status eq 'BANNED') ||
		       ( $status eq 'UNAVAILABLE') ||
		       ( $status eq 'NOT_IN_A_SLOT') ||
		       ( $status eq 'NEEDS_CHECKIN') ||
		       ( $status eq 'WAITING_FOR_GRADE') ||
		       ( $status eq 'INVALID_ACCESS') ||
		       ( &get_version() eq ''))) {
	return 0;
    }
    if ($env{'form.donescreen'}) { return 0; }
    return 1;
}

my @delay;
sub nest { 
    if (@delay) {
	return $delay[-1];
    } else {
	return;
    }
}

sub nested_parse {
    my ($str,$env,$args) = @_;
    my @old_env = @Apache::scripttag::parser_env;
    @Apache::scripttag::parser_env = @$env;
    if (exists($args->{'set_dim_id'})) {
	&enable_dimension_parsing($args->{'set_dim_id'});
    }
    push(@delay,(($args->{'delayed_dim_results'})? 1 : 0));
    my $result = &Apache::scripttag::xmlparse($$str);
    pop(@delay);
    if (exists($args->{'set_dim_id'})) {
	&disable_dimension_parsing();
    }
    @Apache::scripttag::parser_env = @old_env;
    if ($args->{'delayed_dim_results'}) {
	my $dim = &get_dim_id();
	&Apache::lonxml::debug(" tossing out $result ");
	&Apache::lonxml::debug(" usining out $dim 's  ". $dimension{$dim}{'result'});
	return $dimension{$dim}{'result'};
    }
    return $result;
}

sub internal_location {
    my ($id)=@_;
    return '<!-- LONCAPA_INTERNAL_ADD_TASK_STATUS'.$id.' -->';
}

sub submission_time_stamp {
    my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $submissiontime;
    my $version=$Apache::lonhomework::history{'resource.0.version'};
    for (my $v=$Apache::lonhomework::history{'version'};$v>0;$v--) {
	if (defined($Apache::lonhomework::history{$v.':resource.'.$version.'.0.bridgetask.portfiles'})) {
	    $submissiontime=$Apache::lonhomework::history{$v.':timestamp'};
	}
    }
    my $result;
    if ($submissiontime) {
	my $slot_name=$Apache::lonhomework::history{'resource.'.$version.'.0.checkedin.slot'};
	my %slot=&Apache::lonnet::get_slot($slot_name);
	my $diff = $slot{'endtime'} - $submissiontime;
	my ($color,$when)=('#FF6666','after');
	if ($diff > 0) { ($color,$when)=('#336600','before'); }
	my $info;
	if ($diff%60) { $info=($diff%60).' seconds'; }
	$diff=int($diff/60);
	if ($diff%60) { $info=($diff%60).' minutes '.$info; }
	$diff=int($diff/60);
	if ($diff) {    $info=$diff.' hours '.$info; }
	$result='<p><font color="'.$color.'">'.
	    &mt('Student submitted [_1] [_2] the deadline. 
                 (Submission was at [_3], end of period was [_4].)',
		$info,$when,scalar(localtime($submissiontime)),
		scalar(localtime($slot{'endtime'}))).
		'</font></p>';
    }
    return $result;
}

sub file_list {
    my ($files,$uname,$udom) = @_;
    if (!defined($uname) || !defined($udom)) {
	(undef,undef,$udom,$uname) = &Apache::lonxml::whichuser();
    }
    my $file_url = '/uploaded/'.$udom.'/'.$uname.'/portfolio/';

    my $file_list="<ul class=\"LC_GRADING_handininfo\">\n";
    foreach my $partial_file (split(',',$files)) {
	my $file=$file_url.$partial_file;
	$file=~s|/+|/|g;
	&Apache::lonnet::allowuploaded('/adm/bridgetask',$file);
	$file_list.='<li><span style="white-space: nowrap;"><a href="'.$file.'?rawmode=1" target="lonGRDs"><img src="'.
	    &Apache::loncommon::icon($file).'" alt="file icon" border="0" /> '.$file.
	    '</a></span></li>'."\n";
    }
    $file_list.="</ul>\n";
    return $file_list;
}

sub grade_mode {
    if ($env{'form.regrade'} || $env{'form.regradeaspecificsubmission'}) {
	return 'regrade';
    }
    return 'queue_grade';
}

sub webgrade_standard_info {
    my ($version)=&get_version();

    my $file_list = &file_list($Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"});

    my %lt=('done'   => 'Next Item',
	    'stop'   => 'Quit Grading',
	    'fail'   => 'Fail Rest',
	    'cancel' => 'Cancel',
	    );
    %lt=&Apache::lonlocal::texthash(%lt);

    my $result=<<INFO;
  <div class="LC_GRADING_maincontrols">
INFO

    if (&grade_mode() eq 'regrade') {
	$result.=<<INFO;
    <input type="submit" name="cancel" value="$lt{'cancel'}" />
INFO
    }

    $result.=<<INFO;
    <input type="submit" name="next" value="$lt{'done'}" />
    <input type="submit" name="stop" value="$lt{'stop'}" />
    <input type="button" name="fail" value="$lt{'fail'}" 
           onclick="javascript:onFailRest()" />
  </div>
  $file_list
INFO
    return $result;
}

sub done_screen {
    my ($version) = @_;
    my $title=&Apache::lonnet::gettitle();
    my @files=split(',',$Apache::lonhomework::history{'resource.'.$version.'.0.bridgetask.portfiles'});
    my (undef,undef,$domain,$user)= &Apache::lonxml::whichuser();
    my $files = '<ul>';
    my $msg;
    foreach my $file (@files) {
	my $url="/uploaded/$domain/$user/portfolio$file";
	if (! &Apache::lonnet::stat_file($url)) {
	    $file = &mt('<span class="LC_error"> Nonexistent file:</span> '.
			'<span class="LC_filename">[_1]</span>',$file);
	    $msg .= "<p>Submitted non-existant file $file</p>\n";
	} else {
	    $file = '<span class="LC_filename">'.$file.'</span>';
	    $msg .= "<p>Submitted file $file</p>\n";
	}
	$files .= '<li>'.$file.'</li>';
    }
    $files.='</ul>';
    my $subject = "Submission message for $title";
    my ($message_status,$comment_status);
    my $setting = $env{'course.'.$env{'request.course.id'}.'.task_messages'};
    $setting =~ s/^\s*(\S*)\s*$/$1/;
    $setting = lc($setting);
    if ($setting eq 'only_student'
	|| $setting eq 'student_and_user_notes_screen') {
	$message_status =
	    &Apache::lonmsg::user_normal_msg($user,$domain,$subject,$msg);
	$message_status = '<p>'.&mt('Message sent to user: [_1]',
				    $message_status).' </p>';
    }
    if ($setting eq 'student_and_user_notes_screen') {
	$comment_status = 
	    &Apache::lonmsg::store_instructor_comment($subject.'<br />'.
						      $msg,$user,$domain);
	$comment_status = '<p>'.&mt('Message sent to instructor: [_1]',
				    $comment_status).' </p>';
    }
    return <<DONESCREEN;
<h2>$title</h2>
<p> Files submitted: $files </p>
<p> You are now done with this Bridge Task </p>
<hr />
<p> <a href="/adm/logout">Logout</a> </p>
<p> <a href="/adm/roles">Change to a different course</a> </p>
$message_status
$comment_status
DONESCREEN

}

sub start_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

    my ($status,$accessmsg,$slot);
    &Apache::structuretags::init_problem_globals('Task');
    if ($target ne 'webgrade') {
	&Apache::structuretags::initialize_storage();
	&Apache::lonhomework::showhash(%Apache::lonhomework::history);
	if ($env{'request.state'} eq 'construct') {
	    &Apache::structuretags::setup_rndseed($safeeval);
	}
    } 

    $Apache::lonhomework::parsing_a_task=1;

    my $name;
    if ($target eq 'web' || $target eq 'webgrade') {
	$name = &Apache::structuretags::get_resource_name($parstack,$safeeval);
    }

    my ($result,$form_tag_start);
    if ($target eq 'web' || $target eq 'webgrade' || $target eq 'tex'
	|| $target eq 'edit') {
	($result,$form_tag_start) =
	    &Apache::structuretags::page_start($target,$token,$tagstack,
					       $parstack,$parser,$safeeval,
					       $name,&style($target));
	$result .= '<div class="LC_task">'."\n";
    }

    if ($target eq 'web' && $env{'request.state'} ne 'construct') {
	if ($Apache::lonhomework::queuegrade
	    || $Apache::lonhomework::modifygrades) {
	    $result.='<form name="gradesubmission" method="post" action="';
	    my $uri=$env{'request.uri'};
	    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
	    $result.=$uri.'">'.&add_grading_button()."</form>";
	    my $symb=&Apache::lonnet::symbread();
	    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
		$result.='<form method="post" name="slotrequest" action="/adm/slotrequest">'.
		    '<input type="hidden" name="symb" value="'.$symb.'" />'.
		    '<input type="hidden" name="command" value="showslots" />'.
		    '<input type="submit" name="requestattempt" value="'.
		    &mt('Show Slot list').'" />'.
		    '</form>';
		my $target_id = 
		    &Apache::lonstathelpers::make_target_id({symb => $symb,
							     part => '0'});
		$result.='<form method="post" name="gradingstatus" action="/adm/statistics">'.
		    '<input type="hidden" name="problemchoice" value="'.$target_id.'" />'.
		    '<input type="hidden" name="reportSelected" value="grading_analysis" />'.
		    '<input type="submit" name="grading" value="'.
		    &mt('Show Grading Status').'" />'.
		    '</form>';
	    }
	}
    }
    if ($target eq 'web' && $env{'request.state'} eq 'construct') {
	$form_tag_start.=&Apache::structuretags::problem_web_to_edit_header($env{'form.rndseed'});
    }
    if ($target eq 'web' 
	|| ($target eq 'grade' && !$env{'form.webgrade'}) 
	|| $target eq 'answer' 
	|| $target eq 'tex') {
	my ($version,$previous)=&get_version();
	($status,$accessmsg,my $slot_name,$slot) = 
	    &Apache::lonhomework::check_slot_access('0','Task');
	if ($status eq 'CAN_ANSWER' && $version eq '') {
	    # CAN_ANSWER mode, and no current version, unproctored access
	    # thus self-checkedin
	    &check_in('Task',undef,undef,$slot_name);
	    &add_to_queue('gradingqueue',{'type' => 'Task',
					  'time' => time,
					  'slot' => $slot_name});
	    ($version,$previous)=&get_version();
	}

	push(@Apache::inputtags::status,$status);
	$Apache::inputtags::slot_name=$slot_name;
	my $expression='$external::datestatus="'.$status.'";';
	$expression.='$external::gradestatus="'.$Apache::lonhomework::history{"resource.$version.0.solved"}.'";';
	&Apache::run::run($expression,$safeeval);
	&Apache::lonxml::debug("Got $status");
	$result.=&add_previous_version_button($status);
	if (!&show_task($status,$previous)) {
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser,$style);
	    if ( $target eq "web" ) {
		if ($env{'request.state'} eq 'construct') {
		    $result.=$form_tag_start;
		}
		my $msg;
		if ($status eq 'UNAVAILABLE') {
		    $msg.='<h1>'.&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'</h1>';
		} elsif ($status eq 'NOT_IN_A_SLOT') {
		    $msg.='<h1>'.&mt('You are not currently signed up to work at this time and/or place.').'</h1>';
		    $msg.=&add_request_another_attempt_button("Sign up for time to work.");
		} elsif ($status eq 'NEEDS_CHECKIN') {
		    $msg.='<h1>'.&mt('You need the Proctor to validate you.').
			'</h1>'.&proctor_validation_screen($slot);
		} elsif ($status eq 'WAITING_FOR_GRADE') {
		    $msg.='<h1>'.&mt('Your submission is in the grading queue.').'</h1>';
		} elsif ($env{'form.donescreen'}) {
		    $result .= &done_screen($version);
		} elsif ($status ne 'NOT_YET_VIEWED') {
		    $msg.='<h1>'.&mt('Not open to be viewed').'</h1>';
		}
		if ($status eq 'CLOSED' || $status eq 'INVALID_ACCESS') {
		    $msg.='The problem '.$accessmsg;
		}
		$result.=$msg.'<br />';
	    } elsif ($target eq 'tex') {
		$result.='\begin{document}\noindent \vskip 1 mm  \begin{minipage}{\textwidth}\vskip 0 mm';
		if ($status eq 'UNAVAILABLE') {
		    $result.=&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'\vskip 0 mm ';
		} else {
		    $result.=&mt('Problem is not open to be viewed. It')." $accessmsg \\vskip 0 mm ";
		}
	    } elsif ($target eq 'grade' && !$env{'form.webgrade'}) {
		if ($status eq 'NEEDS_CHECKIN') {
		    if(&proctor_check_auth($slot_name,$slot,'Task')
		       && defined($Apache::inputtags::slot_name)) {
			my $result=
			    &add_to_queue('gradingqueue',
					  {'type' => 'Task',
					   'time' => time,
					   'slot' => 
					       $Apache::inputtags::slot_name});
			&Apache::lonxml::debug("add_to_queue said $result");
		    }
		}
	    }
	} elsif ($target eq 'web') {

	    $result.=&preserve_grade_info();
	    $result.=&internal_location();
	    $result.=$form_tag_start.
		'<input type="hidden" name="submitted" value="yes" />';
	    &Apache::lonxml::startredirection();
	}
    } elsif ( ($target eq 'grade' && $env{'form.webgrade'}) ||
	      $target eq 'webgrade') {
	my $webgrade='yes';
	if ($target eq 'webgrade') {
	    $result.= "\n".'<div class="LC_GRADING_task">'."\n".
		'<script type="text/javascript" 
                         src="/res/adm/includes/task_grading.js"></script>';
	    #$result.='<br />Review'.&show_queue('reviewqueue');
	    #$result.='<br />Grade'.&show_queue('gradingqueue');
	}
	# FIXME Blast! still need to reorg this, need to reshow the
        #       queue being reviewed once done with the grade pass...
        #       Hrrm, vaildation pass should perhaps say 'not_locked'
        #       perhaps do a search if there is a key that is mine and if
        #       there isn't reshow the queue....
	my ($todo,$status_code,$msg)=&get_key_todo($target);

	if ($todo) {
	    &setup_env_for_other_user($todo,$safeeval);
	    my ($symb,$uname,$udom)=&decode_queue_key($todo);
	    $result.="\n".'<table><tr><td>Found '.
		&Apache::lonnet::gettitle($symb).' for '.$uname.' at '.$udom.'</td></tr></table>';
	    $form_tag_start.=
		'<input type="hidden" name="gradingkey" value="'.
		&escape($todo).'" />';
	    $Apache::bridgetask::queue_key=$todo;
	    &Apache::structuretags::initialize_storage();
	    &Apache::lonhomework::showhash(%Apache::lonhomework::history);
	    if ($target eq 'webgrade' && $status_code eq 'selected') {
		$form_tag_start.=
		    '<input type="hidden" name="queuemode" value="selected" />';
	    }
	} else {
	    if ($target eq 'webgrade') {
		$result.="\n";
		my $back='<p><a href="/adm/flip?postdata=return:">'.
		    &mt('Return to resource').'</a></p>';
		if      ($status_code eq 'stop') {
		    $result.='<b>'.&mt("Stopped grading.").'</b>'.$back;
		} elsif ($status_code eq 'cancel') {
		    $result.='<b>'.&mt("Cancelled grading.").'</b>'.$back;
		} elsif ($status_code eq 'never_versioned') {
		    $result.='<b>'.
			&mt("Requested user has never accessed the task.").
			'</b>'.$back;
		} elsif ($status_code =~ /still_open:(.*)/) {
		    my $date = &Apache::lonlocal::locallocaltime($1);
		    $result.='<b>'.
			&mt("Task is still open, will close at [_1].",$date).
			'</b>'.$back;
		} elsif ($status_code eq 'lock_failed') {
		    $result.='<b>'.&mt("Failed to lock the requested record.")
			.'</b>'.$back;
		} elsif ($status_code eq 'unlock') {
		    $result.='<b>'.&mt("Unlocked the requested record.")
			.'</b>'.$back;
		    $result.=&show_queue($env{'form.queue'},1);
		} elsif ($status_code eq 'show_list') {
		    $result.=&show_queue($env{'form.queue'},1);
		} elsif ($status_code eq 'select_user') {
		    $result.=&select_user();
		} elsif ($status_code eq 'unable') {
		    $result.='<b>'.&mt("Unable to aqcuire a user to grade.").'</b>'.$back;
		} elsif ($status_code eq 'not_allowed') {
		    $result.='<b>'.&mt('Not allowed to grade the requested user.').' '.$msg.'</b>'.$back;
		} else {
		    $result.='<b>'.&mt("No user to be graded.").'</b>'.$back;
		}
	    }
	    $webgrade='no';
	}
	if (!$todo || $env{'form.cancel'}) {
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser,$style);
	}
	if ($target eq 'webgrade' && defined($env{'form.queue'})) {
	    if ($webgrade eq 'yes') {
		$result.=&submission_time_stamp();
	    }
	    $result.=$form_tag_start;
	    $result.='<input type="hidden" name="webgrade" value="'.
		$webgrade.'" />';
	    $result.='<input type="hidden" name="queue" value="'.
		$env{'form.queue'}.'" />';
	    if ($env{'form.regrade'}) {
		$result.='<input type="hidden" name="regrade" value="'.
		    $env{'form.regrade'}.'" />';
	    }
	    if ($env{'form.chosensections'}) {
		my @chosen_sections=
		    &Apache::loncommon::get_env_multiple('form.chosensections');
		foreach my $sec (@chosen_sections) {
		    $result.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
		}
	    }
	    if ($webgrade eq 'yes') { $result.=&webgrade_standard_info(); }
	}
	if ($target eq 'webgrade') {
	    $result.="\n".'<div id="LC_GRADING_criterialist">';
	}
    } elsif ($target eq 'edit') {
	$result.=$form_tag_start.
	    &Apache::structuretags::problem_edit_header();
	$Apache::lonxml::warnings_error_header=
	    &mt("Editor Errors - these errors might not effect the running of the problem, but they will likely cause problems with further use of the Edit mode. Please use the EditXML mode to fix these errors.")."<br />";
	my $temp=&Apache::edit::insertlist($target,$token);
	$result.=$temp;
    } else {
	# page_start returned a starting result, delete it if we don't need it
	$result = '';
    }
    return $result;
}

sub get_task_end_time {
    my ($queue_entry,$symb,$udom,$uname) = @_;

    my $end_time;
    if (my $slot = &slotted_access($queue_entry)) {
	my %slot_data=&Apache::lonnet::get_slot($slot);
	$end_time = $slot_data{'endtime'};
    } else {
	$end_time = &Apache::lonhomework::due_date('0',$symb,
						   $udom,$uname);
    }
    return $end_time;
}

sub get_key_todo {
    my ($target)=@_;
    my $todo;

    if (defined($env{'form.reviewasubmission'})) {
	&Apache::lonxml::debug("review a submission....");
	$env{'form.queue'}='reviewqueue';
	return (undef,'show_list');
    }

    if (defined($env{'form.reviewagrading'})) {
	&Apache::lonxml::debug("review a grading....");
	$env{'form.queue'}='gradingqueue';
	return (undef,'show_list');
    }

    if (defined($env{'form.regradeasubmission'})) {
	&Apache::lonxml::debug("regrade a grading....");
	$env{'form.queue'}='none';
	return (undef,'select_user');
    }


    my $me=$env{'user.name'}.':'.$env{'user.domain'};

    #need to try both queues..
    if (defined($env{'form.regradeaspecificsubmission'}) &&
	defined($env{'form.gradinguser'})               &&
	defined($env{'form.gradingdomain'})               ) {
	my ($symb,$cid)=&Apache::lonxml::whichuser();
	my $cnum  = $env{'course.'.$cid.'.num'};
	my $cdom  = $env{'course.'.$cid.'.domain'};
	my $uname = $env{'form.gradinguser'};
	my $udom  = $env{'form.gradingdomain'};

	my $gradingkey=&encode_queue_key($symb,$udom,$uname);

	my $queue;

	if      (&in_queue('gradingqueue',$symb,$cdom,$cnum,$udom,$uname)) {
	    $env{'form.queue'} = $queue = 'gradingqueue';
	} elsif (&in_queue('reviewqueue' ,$symb,$cdom,$cnum,$udom,$uname)) {
	    $env{'form.queue'} = $queue = 'reviewqueue';
	}
	
	if (!$queue) {
	    $env{'form.queue'} = $queue = 'none';
	    #not queued so doing either a re or pre grade
	    my %status = &Apache::lonnet::restore($symb,$cid,$udom,$uname);
	    if ($status{'resource.0.version'} < 1) {
		return (undef,'never_versioned');
	    }
	    return ($gradingkey);
	}

	if ($queue) {
	    my $queue_entry = &get_queue_data($queue,$udom,$uname);
	
	    my $end_time = &get_task_end_time($queue_entry,$symb,
					      $udom,$uname);
	    if ($end_time > time) {
		return (undef,"still_open:$end_time");
	    }
	}

	my $who=&queue_key_locked($queue,$gradingkey);
	if ($who eq $me) {
	    #already have the lock
	    $env{'form.gradingkey'}=&escape($gradingkey);
	    &Apache::lonxml::debug("already locked");
	    return ($gradingkey);
	}
	
	if (!defined($who)) {
	    if (&lock_key($queue,$gradingkey)) {
		&Apache::lonxml::debug("newly locked");
		return ($gradingkey);
	    } else {
		return (undef,'lock_failed');
	    }
	}

	#otherwise (defined($who) && $who ne $me) some else has it...
	return (undef,'not_allowed',
		&mt('Another user ([_1]) currently has the record for [_2] locked.',
		    $who,$env{'form.gradinguser'}.':'.$env{'form.gradingdomain'}));
    }


    my $queue=$env{'form.queue'};

    if (!defined($queue)) {
	$env{'form.queue'}=$queue='gradingqueue';
    }

    my $gradingkey=&unescape($env{'form.gradingkey'});

    if ($env{'form.queue'} eq 'none') {
	if (defined($env{'form.gradingkey'})) {
	    if ($target eq 'webgrade') {
		if ($env{'form.stop'}) {
		    return (undef,'stop');
		} elsif ($env{'form.cancel'}) {
		    return (undef,'cancel');
		} elsif ($env{'form.next'}) {
		    return (undef,'select_user');
		}
	    }
	    return ($gradingkey,'selected');
	} else {
	    return (undef,'select_user');
	}
    }
    if (defined($env{'form.queue'}) && defined($env{'form.gradingkey'})
	&& !defined($env{'form.gradingaction'}) 
	&& $env{'form.queuemode'} eq 'selected') {
	
	my $who=&queue_key_locked($queue,$gradingkey);
	if ($who eq $me) {
	    &Apache::lonxml::debug("Found a key was given to me");
	    return ($gradingkey,'selected');
	} else {
	    return (undef,'show_list');
	}

    }

    if ($target eq 'webgrade' && $env{'form.queuemode'} eq 'selected') {
	if ($env{'form.gradingaction'} eq 'resume') {
	    delete($env{'form.gradingaction'});
	    &Apache::lonxml::debug("Resuming a key");
	    return ($gradingkey);
	} elsif ($env{'form.gradingaction'} eq 'unlock') {
	    &Apache::lonxml::debug("Unlocking a key ".
				     &check_queue_unlock($queue,$gradingkey,1));
	    return (undef,'unlock');
	} elsif ($env{'form.gradingaction'} eq 'select') {
	    &Apache::lonxml::debug("Locking a key");
	    if (&lock_key($queue,$gradingkey)) {
		&Apache::lonxml::debug("Success $queue");
		return ($gradingkey);
	    }
	    &Apache::lonxml::debug("Failed $queue");
	    return (undef,'lock_failed');
	}
    }

    if ($env{'form.queuemode'} ne 'selected') {
	# don't get something new from the queue if they hit the stop button
    	if (!(($env{'form.cancel'} || $env{'form.stop'}) 
	      && $target eq 'webgrade') 
	    && !$env{'form.gradingaction'}) {
	    &Apache::lonxml::debug("Getting anew $queue");
	    return (&get_from_queue($queue));
	} else {
	    return (undef,'stop');
	}
    }
    return (undef,undef)
}

sub minimize_storage {
    foreach my $key (keys(%Apache::lonhomework::results)) {
	if ($key =~ /regrader$/) { next; }
	if ($Apache::lonhomework::results{$key} eq
	    $Apache::lonhomework::history{$key}) {
	    delete($Apache::lonhomework::results{$key});
	}
    }
}

sub end_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result='';
    my $status=$Apache::inputtags::status['-1'];
    my ($version,$previous)=&get_version();
    if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' ||
	$target eq 'tex') {
	if ($target eq 'web' || $target eq 'answer' || $target eq 'tex') {
	    if ($target eq 'web') {
		if (&show_task($status,$previous)) {
		    $result.=&Apache::lonxml::endredirection();
		}
		if ($status eq 'CAN_ANSWER' && !$previous && 
		    !$env{'form.donescreen'}) {
		    $result.="\n".'<table border="1">'.
			&Apache::inputtags::file_selector("$version.0",
							  "bridgetask","*",
							  'portfolioonly',
							  '
<h2>'.&mt('Submit Portfolio Files for Grading').'</h2>
<p>'.&mt('Indicate the files from your portfolio to be evaluated in grading this task.').'</p>').
							  "</table>";
		}
		if (!$previous && $status ne 'SHOW_ANSWER' &&
		    &show_task($status,$previous)) {
		    $result.=&Apache::inputtags::gradestatus('0');
		    $result.='</form>';
		    my $action = &Apache::lonenc::check_encrypt($env{'request.uri'});
		    $result.=<<DONEBUTTON;
<form name="done" method="post" action="$action">
   <input type="hidden" name="donescreen" value="1" />
   <input type="submit" value="Done" />
</form>
DONEBUTTON
                }
		if (&show_task($status,$previous) &&
		    $Apache::lonhomework::history{"resource.$version.0.status"} =~ /^(pass|fail)$/) {
		    my $bt_status=$Apache::lonhomework::history{"resource.$version.0.status"};
		    my $title=&Apache::lonnet::gettitle();

		    my $start_time;

		    my $slot_name=
			$Apache::lonhomework::history{"resource.$version.0.checkedin.slot"};
		    if ($slot_name) {
			my %slot=&Apache::lonnet::get_slot($slot_name);

			$start_time=$slot{'starttime'}
		    } else {
			$start_time= 
			    &Apache::lonnet::EXT('resource.0.opendate');
		    }
		    $start_time=&Apache::lonlocal::locallocaltime($start_time);

		    my $status = "\n<div class='LC_$bt_status LC_criteria'>\n";
		    
		    if ($bt_status eq 'pass')  {
			$status.='<h2>You passed the '.$title.' given on '.
			    $start_time.'</h2>';
		    }
		    if ($bt_status eq 'fail')  {
			$status.='<h2>You did not pass the '.$title.' given on '.
			    $start_time.'</h2>';
			if (!$previous) {
			    $status.=&add_request_another_attempt_button();
			}
		    }
		    my $man_count=0;
		    my $opt_count=0;
		    my $opt_passed=0;
		    foreach my $dim (keys(%Apache::bridgetask::top_dimensionlist)) {
			if ($Apache::bridgetask::top_dimensionlist{$dim}{'manadatory'}
			    eq 'N') {
			    $opt_count++;
			    if ($Apache::lonhomework::history{"resource.$version.0.$dim.status"} eq 'pass') {
				$opt_passed++;
			    }
			} else {
			    $man_count++;
			}
		    }
		    
		    my $opt_req=&Apache::lonxml::get_param('OptionalRequired',
							 $parstack,$safeeval);
		    if ($opt_req !~ /\S/) { $opt_req='0'; }
		    $status.="\n<p>".&mt('You needed to pass all of the [_1]  mandatory components and [_2] of the [_3] optional components, of which you passed [_4].',$man_count,$opt_req,$opt_count,$opt_passed)."</p></div>\n";

		    my $internal_location=&internal_location();
		    $result=~s/\Q$internal_location\E/$status/;
		}
		$result.="\n</div>\n".
		    &Apache::loncommon::end_page({'discussion' => 1});
	    }
	}

	my $useslots = &Apache::lonnet::EXT("resource.0.useslots");
	my %queue_data = ('type' => 'Task',
			  'time' => time,);
	if (defined($Apache::inputtags::slot_name)) {
	    $queue_data{'slot'} = $Apache::inputtags::slot_name;
	} elsif (defined($Apache::lonhomework::history{"resource.$version.0.checkedin.slot"})) {
	    $queue_data{'slot'} = $Apache::lonhomework::history{"resource.$version.0.checkedin.slot"};
	}
	

	if ($target eq 'grade' && !$env{'form.webgrade'} && !$previous) {
	    my $award='SUBMITTED';
	    &Apache::essayresponse::file_submission("$version.0",'bridgetask',
						    'portfiles',\$award);
	    if ($award eq 'SUBMITTED' &&
		$Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}) {
		$Apache::lonhomework::results{"resource.0.tries"}=
		    $Apache::lonhomework::results{"resource.$version.0.tries"}=
		    1+$Apache::lonhomework::history{"resource.$version.0.tries"};

		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.0.award"}=
		    $award;
		$Apache::lonhomework::results{"resource.0.submission"}=
		    $Apache::lonhomework::results{"resource.$version.0.submission"}='';
	    } else {
		delete($Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"});
		$award = '';
	    }
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    &Apache::structuretags::finalize_storage();
	    if ($award eq 'SUBMITTED') {
		&add_to_queue('gradingqueue',\%queue_data);
	    }
	} elsif ($Apache::lonhomework::results{'INTERNAL_store'}) {
	    &Apache::structuretags::finalize_storage();
	}
	if ($target eq 'grade' && $env{'form.webgrade'} eq 'yes' 
	    && exists($env{'form.cancel'})) {
	    &check_queue_unlock($env{'form.queue'});
	    &Apache::lonxml::debug(" cancelled grading .".$env{'form.queue'});
	} elsif ($target eq 'grade' && $env{'form.webgrade'} eq 'yes' 
		 && !exists($env{'form.cancel'})) {
	    my $optional_required=
		&Apache::lonxml::get_param('OptionalRequired',$parstack,
					   $safeeval);
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;   
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    foreach my $dim (keys(%Apache::bridgetask::top_dimensionlist)) {
		my $status=
		    $Apache::lonhomework::results{"resource.$version.0.$dim.status"};
		my $mandatory=
		    ($Apache::bridgetask::top_dimensionlist{$dim}{'manadatory'} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} elsif ($status eq 'review') {
		    $review++;
		} else {
		    $ungraded++;
		}
	    }
	    if ($optional_passed < $optional_required) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all dim ".join(':',keys(%Apache::bridgetask::top_dimensionlist))."results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    $Apache::lonhomework::results{'resource.0.regrader'}=
		$env{'user.name'}.':'.$env{'user.domain'};
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='review';
		if ($env{'form.queue'} eq 'reviewqueue') {
		    &check_queue_unlock($env{'form.queue'});
		    &Apache::lonxml::debug(" still needs review not changing status.");
		} else {
		    if ($env{'form.queue'} ne 'none') {
			&move_between_queues($env{'form.queue'},'reviewqueue');
		    } else {
			&add_to_queue('reviewqueue',\%queue_data);
		    }
		}
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='ungraded';
		if ($env{'form.queue'} eq 'reviewqueue') {
		    &Apache::lonxml::debug("moving back.");
		    &move_between_queues($env{'form.queue'},
					 'gradingqueue');
		} elsif ($env{'form.queue'} eq 'none' ) {
		    &add_to_queue('gradingqueue',\%queue_data);	
		} else {
		    &check_queue_unlock($env{'form.queue'});
		}
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='fail';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='incorrect_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='INCORRECT';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='0';
		&remove_from_queue($env{'form.queue'}); 

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		
		if ($env{'form.regrade'} ne 'yes') {
		    $Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
			$Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		    &Apache::grades::version_portfiles(
						       \%Apache::lonhomework::results,
						       ["$version.0.bridgetask"],$courseid,
						       $symb,$udom,$uname,
						       ["$version.0.bridgetask"]);
		}
	    } else {
		$Apache::lonhomework::results{"resource.$version.0.status"}='pass';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='correct_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='EXACT_ANS';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='1';
		&remove_from_queue($env{'form.queue'});

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		if ($env{'form.regrade'} ne 'yes') {
		    $Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
			$Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		    &Apache::grades::version_portfiles(
						       \%Apache::lonhomework::results,
						       ["$version.0.bridgetask"],$courseid,
						       $symb,$udom,$uname,
						       ["$version.0.bridgetask"]);
		}
	    }
	    $Apache::lonhomework::results{"resource.0.status"}=
		$Apache::lonhomework::results{"resource.$version.0.status"};
	    if (defined($Apache::lonhomework::results{"resource.$version.0.awarded"})) {
		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.0.award"};
		$Apache::lonhomework::results{"resource.0.awarded"}=
		    $Apache::lonhomework::results{"resource.$version.0.awarded"};
		$Apache::lonhomework::results{"resource.0.solved"}=
		    $Apache::lonhomework::results{"resource.$version.0.solved"};
	    }
	    &minimize_storage();
	    &Apache::structuretags::finalize_storage();
	}
    } elsif ($target eq 'webgrade') {
	$result.="</div>";
	#$result.='<input type="submit" name="next" value="'.
	#    &mt('Save &amp; Next').'" /> ';
	#$result.='<input type="submit" name="end" value="'.
	#    &mt('Save &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="throwaway" value="'.
	#    &mt('Throw Away &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="save" value="'.
	#    &mt('Save Partial Grade and Continue Grading').'" /> ';
	$result.='</form>'."\n</div>\n</div>\n".
	    &Apache::loncommon::end_page();
    } elsif ($target eq 'meta') {
	$result.=&Apache::response::meta_package_write('Task');
        $result.=&Apache::response::meta_stores_write('solved','string',
						      'Problem Status');
	$result.=&Apache::response::meta_stores_write('tries','int_zeropos',
						      'Number of Attempts');
	$result.=&Apache::response::meta_stores_write('awarded','float',
						      'Partial Credit Factor');
	$result.=&Apache::response::meta_stores_write('status','string',
						      'Bridge Task Status');
    }
    &Apache::structuretags::reset_problem_globals('Task');
    undef($Apache::lonhomework::parsing_a_task);
    return $result;
}

sub move_between_queues {
    my ($src_queue,$dest_queue)=@_;
    my $cur_data;
    if ($src_queue ne 'none') {
	$cur_data=&get_queue_data($src_queue);
	if (!$cur_data) { return 'not_exist'; }
    } else {
	$cur_data = ['none'];
    }
    my $result=&add_to_queue($dest_queue,$cur_data);
    if ($result ne 'ok') {
	return $result;
    }
    &check_queue_unlock($src_queue);
    return &remove_from_queue($src_queue);
}

sub check_queue_unlock {
    my ($queue,$key,$allow_not_me)=@_;
    if ($queue eq 'none') { return 'ok'; }
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    if (!defined($key)) {
	$key="$symb\0queue\0$uname:$udom";
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $me=$env{'user.name'}.':'.$env{'user.domain'};
    my $who=&queue_key_locked($queue,$key,$cdom,$cnum);
    if  ($who eq $me) {
	&Apache::lonxml::debug("unlocking my own $who");
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    } elsif ($allow_not_me) {
	&Apache::lonxml::debug("unlocking $who by $me");
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    }
    return 'not_owner';
}

sub in_queue {
    my ($queue,$symb,$cdom,$cnum,$udom,$uname)=@_;
    if ($queue eq 'none') { return 0; }
    if (!defined($symb) || !defined($cdom) || !defined($cnum)
	|| !defined($udom) || !defined($uname)) {
	($symb,my $cid,$udom,$uname)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }

    my $key=&encode_queue_key($symb,$udom,$uname);
    my %results = &Apache::lonnet::get($queue,[$key],$cdom,$cnum);

    if (defined($results{$key})) {
	return 1;
    }
    return 0;
}

sub remove_from_queue {
    my ($queue,$symb,$cdom,$cnum,$udom,$uname)=@_;
    if ($queue eq 'none') { return 'ok'; }
    if (!defined($symb) || !defined($cdom) || !defined($cnum)
	|| !defined($udom) || !defined($uname)) {
	($symb,my $cid,$udom,$uname)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    if (!&in_queue($queue,$symb,$cdom,$cnum,$udom,$uname)) {
	return 'ok';
    }
    my $key=&encode_queue_key($symb,$udom,$uname);
    my @keys=($key,"$key\0locked");
    return &Apache::lonnet::del($queue,\@keys,$cdom,$cnum);
}

sub setup_env_for_other_user {
    my ($queue_key,$safeeval)=@_;
    my ($symb,$uname,$udom)=&decode_queue_key($queue_key);
    &Apache::lonxml::debug("setup_env for $queue_key");
    $env{'form.grade_symb'}=$symb;
    $env{'form.grade_domain'}=$udom;
    $env{'form.grade_username'}=$uname;
    $env{'form.grade_courseid'}=$env{'request.course.id'};
    &Apache::lonxml::initialize_rndseed($safeeval);
}

sub get_queue_data {
    my ($queue,$udom,$uname)=@_;
    my ($symb,$cid,$other_udom,$other_uname)=&Apache::lonxml::whichuser();
    if (!$uname || !$udom) {
	$uname=$other_uname;
	$udom =$other_udom;
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo="$symb\0queue\0$uname:$udom";
    my ($key,$value)=&Apache::lonnet::get($queue,[$todo],$cdom,$cnum);
    if ($key eq $todo && ref($value)) {
	return $value;
    }
    return undef;
}


sub check_queue_for_key {
    my ($cdom,$cnum,$queue,$todo)=@_;

    my %results=
	&Apache::lonnet::get($queue,[$todo,"$todo\0locked"],$cdom,$cnum);
    
    if (exists($results{$todo}) && ref($results{$todo})) {
	if (defined($results{"$todo\0locked"})) {
	    return 'locked';
	}
	if (my $slot=&slotted_access($results{$todo})) {
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if ($slot_data{'endtime'} > time) { 
		return 'in_progress';
	    }
	} else {
	    my ($symb) = &decode_queue_key($todo);
	    my $due_date = &Apache::lonhomework::due_date('0',$symb);
	    if ($due_date > time) {
		return 'in_progress';
	    }
	}
	return 'enqueued';
    }
    return undef;
}

sub add_to_queue {
    my ($queue,$user_data)=@_;
    if ($queue eq 'none') { return 'ok'; }
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    if (!$cid || $env{'request.state'} eq 'construct') {
	return 'no_queue';
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my %data;
    $data{"$symb\0queue\0$uname:$udom"}=$user_data;
    return &Apache::lonnet::cput($queue,\%data,$cdom,$cnum);
}

sub get_limited_classlist {
    my ($sections) = @_;

    my $classlist = &Apache::loncoursedata::get_classlist();
    foreach my $student (keys(%$classlist)) {
	if ( $classlist->{$student}[&Apache::loncoursedata::CL_STATUS()]
	     ne 'Active') {
	    delete($classlist->{$student});
       	}
    }

    if (ref($sections) && !grep('all',@{ $sections })) {
	foreach my $student (keys(%$classlist)) {
	    my $section  = 
		$classlist->{$student}[&Apache::loncoursedata::CL_SECTION()];
	    if (! grep($section,@{ $sections })) {
		delete($classlist->{$student});
	    }
	}
    }
    return $classlist;
}


sub show_queue {
    my ($queue,$with_selects)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};

    my @chosen_sections=
	&Apache::loncommon::get_env_multiple('form.chosensections');

    my $classlist = &get_limited_classlist(\@chosen_sections);

    if (!(grep(/^all$/,@chosen_sections))) {
	$result.='<p> Showing only sections <tt>'.join(', ',@chosen_sections).
	    '</tt>.</p> '."\n";
    }

    my ($view,$view_section);
    my $scope = $env{'request.course.id'};
    if (!($view=&Apache::lonnet::allowed('vgr',$scope))) {
	$scope .= '/'.$env{'request.course.sec'};
	if ( $view = &Apache::lonnet::allowed('vgr',$scope)) {
	    $view_section=$env{'request.course.sec'};
	} else {
	    undef($view);
	}
    }

    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return "\n<h3>Current Queue - $queue</h3>".
	    &Apache::loncommon::start_data_table().
	    &Apache::loncommon::start_data_table_row().
	    '<td>'.&mt('Empty').'</td>'.
	    &Apache::loncommon::end_data_table_row().
	    &Apache::loncommon::end_data_table();
    }
    my $title=&Apache::lonnet::gettitle($symb);
    $result.="\n<h3>Current Queue - $title $queue </h3>".
	&Apache::loncommon::start_data_table().
	&Apache::loncommon::start_data_table_header_row();
    if ($with_selects) { $result.="<th>Status</th><th></th>"; }
    $result.="<th>user</th><th>data</th>".
	&Apache::loncommon::end_data_table_header_row();
    foreach my $key (sort(keys(%queue))) {
	my ($symb,$uname,$udom) = &decode_queue_key($key);
	if (!defined($classlist->{$uname.':'.$udom})) { next; }
	
	my $section = $classlist->{$uname.':'.$udom}[&Apache::loncoursedata::CL_SECTION()];

	my $can_view=1;
	if (!$view
	    || ($view_section && !$section)
	    || ($view_section && $section && ($view_section ne $section))) {
	    $can_view=0;
	}

	if ($key=~/locked$/ && !$with_selects) {
	    $result.= &Apache::loncommon::start_data_table_row().
		"<td>$uname</td>";
	    $result.='<td>'.$queue{$key}.'</td></tr>';
	} elsif ($key=~/timestamp$/ && !$with_selects) {
	    $result.=&Apache::loncommon::start_data_table_row()."<td></td>";
	    $result.='<td>'.
		&Apache::lonlocal::locallocaltime($queue{$key})."</td></tr>";
	} elsif ($key!~/(timestamp|locked)$/) {
	    $result.= &Apache::loncommon::start_data_table_row();
	    my ($end_time,$slot_text);
	    if (my $slot=&slotted_access($queue{$key})) {
		my %slot_data=&Apache::lonnet::get_slot($slot);
		$end_time = $slot_data{'endtime'};
		$slot_text = &mt('Slot: [_1]',$slot);
	    } else {
		$end_time = &Apache::lonhomework::due_date('0',$symb);
		$slot_text = '';
	    }
	    if ($with_selects) {
		my $ekey=&escape($key);
		my ($action,$description,$status)=('select',&mt('Select'));
		if (exists($queue{"$key\0locked"})) {
		    my $me=$env{'user.name'}.':'.$env{'user.domain'};
		    $status=&mt('Locked by <tt>[_1]</tt>',$queue{"$key\0locked"});
		    if ($me eq $queue{"$key\0locked"}) {
			($action,$description)=('resume',&mt('Resume'));
		    } else {
			($action,$description)=('unlock',&mt('Unlock'));
		    }
		}
		my $seclist;
		foreach my $sec (@chosen_sections) {
		    $seclist.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
		}
		if ($can_view && ($end_time ne '' && time > $end_time)) {
		    $result.=(<<FORM);
<td>$status</td>
<td>
<form style="display: inline" method="post">
 <input type="hidden" name="gradingkey" value="$ekey" />
 <input type="hidden" name="queue" value="$queue" />
 <input type="hidden" name="gradingaction" value="$action" />
 <input type="hidden" name="webgrade" value="no" />
 <input type="hidden" name="queuemode" value="selected" />
 <input type="submit" name="submit" value="$description" />
 $seclist
</form>
</td>
FORM
                } elsif (!$can_view && ($end_time ne '' && time > $end_time)) {
		    $result.='<td>'.&mt("Not gradable").'</td><td>&nbsp;</td>'
                } else {
		    $result.='<td>'.&mt("In Progress").'</td><td>&nbsp;</td>'
		}
	    }
	    $result.= "<td>".$classlist->{$uname.':'.$udom}[&Apache::loncoursedata::CL_FULLNAME()].
		" <tt>($uname:$udom)</tt> </td>";
	    $result.='<td>'.$slot_text.' End time: '.
		&Apache::lonlocal::locallocaltime($end_time).
		"</td>".&Apache::loncommon::end_data_table_row();
	}
    }
    $result.= &Apache::loncommon::end_data_table()."<hr />\n";
    return $result;
}

sub get_queue_counts {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};

    my $classlist=&get_limited_classlist();

    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return (0,0,0);
    }
    my ($entries,$ready_to_grade,$locks)=(0,0,0);
    my %slot_cache;
    foreach my $key (sort(keys(%queue))) {
	my ($symb,$uname,$udom) = &decode_queue_key($key);
	if (!defined($classlist->{$uname.':'.$udom})) { next; }

	if ($key=~/locked$/) {
	    $locks++;
	} elsif ($key=~/timestamp$/) {
	    #ignore
	} elsif ($key!~/(timestamp|locked)$/) {
	    $entries++;
	    if (my $slot=&slotted_access($queue{$key})) {
		if (!exists($slot_cache{$slot})) {
		    my %slot_data=&Apache::lonnet::get_slot($slot);
		    $slot_cache{$slot} = \%slot_data;
		}
		if (time > $slot_cache{$slot}{'endtime'}) {
		    $ready_to_grade++;
		}
	    } else {
		my $due_date = &Apache::lonhomework::due_date('0',$symb);
		if ($due_date ne '' && time > $due_date) {
		    $ready_to_grade++;
		}
	    }
	}
    }
    return ($entries,$ready_to_grade,$locks);
}

sub encode_queue_key {
    my ($symb,$udom,$uname)=@_;
    return "$symb\0queue\0$uname:$udom";
}

sub decode_queue_key {
    my ($key)=@_;
    my ($symb,undef,$user) = split("\0",$key);
    my ($uname,$udom) = split(':',$user);
    return ($symb,$uname,$udom);
}

sub queue_key_locked {
    my ($queue,$key,$cdom,$cnum)=@_;
    if (!defined($cdom) || !defined($cnum)) {
	my (undef,$cid)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    my ($key_locked,$value)=
	&Apache::lonnet::get($queue,["$key\0locked"],$cdom,$cnum);
    if ($key_locked eq "$key\0locked") {
	return $value;
    }
    return undef;
}

sub slotted_access {
    my ($queue_entry) = @_;
    if (ref($queue_entry) eq 'ARRAY') {
	if (defined($queue_entry->[0])) {
	    return $queue_entry->[0];
	}
	return undef;
    } elsif (ref($queue_entry) eq 'HASH') {
	if (defined($queue_entry->{'slot'})) {
	    return $queue_entry->{'slot'};
	}
	return undef;
    }
    return undef;
}

sub pick_from_queue_data {
    my ($queue,$check_section,$queuedata,$cdom,$cnum,$classlist)=@_;
    my @possible; # will hold queue entries that are valid to be selected
    foreach my $key (keys(%$queuedata)) {
	if ($key =~ /\0locked$/) { next; }
	if ($key =~ /\0timestamp$/) { next; }

	my ($symb,$uname,$udom)=&decode_queue_key($key);
	if (!defined($classlist->{$uname.':'.$udom})) { next; }

	if ($check_section) {
	    my $section =
		$classlist->{$uname.':'.$udom}[&Apache::loncoursedata::CL_SECTION()];
	    if ($section eq $check_section) {
		&Apache::lonxml::debug("my sec");
		next;
	    }
	}
	my $end_time;
	if (my $slot=&slotted_access($queuedata->{$key})) {
	    &Apache::lonxml::debug("looking at slot $slot");
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if ($slot_data{'endtime'} < time) { 
		$end_time = $slot_data{'endtime'};
	    } else {
		&Apache::lonxml::debug("not time ".$slot_data{'endtime'});
		next;
	    }
	} else {
	    my $due_date = &Apache::lonhomework::due_date('0',$symb);
	    if ($due_date < time) {
		$end_time = $due_date;
	    } else {
		&Apache::lonxml::debug("not time $due_date");
		next;
	    }
	}
	
	if (exists($queuedata->{"$key\0locked"})) {
	    &Apache::lonxml::debug("someone already has um.");
	    next;
	}
	push(@possible,[$key,$end_time]);
    }
    if (@possible) {
        # sort entries in order by slot end time
	@possible = sort { $a->[1] <=> $b->[1] } @possible;
	# pick one of the entries in the top 10% in small queues and one
	# of the first ten entries in large queues
	#my $ten_percent = int($#possible * 0.1);
	#if ($ten_percent < 1 ) { $ten_percent = 1;  }
	#if ($ten_percent > 10) { $ten_percent = 10; }
	#my $max=($#possible < $ten_percent) ? $#possible : $ten_percent;
	
	#return $possible[int(rand($max))][0];
	return $possible[0][0];
    }
    return undef;
}

sub find_mid_grade {
    my ($queue,$symb,$cdom,$cnum)=@_;
    my $todo=&unescape($env{'form.gradingkey'});
    my $me=$env{'user.name'}.':'.$env{'user.domain'};
    if ($todo) {
	my $who=&queue_key_locked($queue,$todo,$cdom,$cnum);
	if ($who eq $me) { return $todo; }
    }
    my $regexp="^$symb\0.*\0locked\$";
    my %locks=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    foreach my $key (keys(%locks)) {
	my $who=$locks{$key};
	if ($who eq $me) {
	    $todo=$key;
	    $todo=~s/\0locked$//;
	    return $todo;
	}
    }
    return undef;
}

sub lock_key {
    my ($queue,$todo)=@_;
    my $me=$env{'user.name'}.':'.$env{'user.domain'};
    my (undef,$cid)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $success=&Apache::lonnet::newput($queue,{"$todo\0locked"=> $me},
					$cdom,$cnum);
    &Apache::lonxml::debug("success $success $todo");
    if ($success eq 'ok') {
	return 1;
    }
    return 0;
}

sub get_queue_symb_status {
    my ($queue,$symb,$cdom,$cnum) = @_;
    if (!defined($cdom) || !defined($cnum)) {
	my (undef,$cid)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    my $classlist=&get_limited_classlist();

    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) { return; }
    my @users;
    foreach my $key (sort(keys(%queue))) {
	next if ($key=~/locked$/);
	next if ($key=~/timestamp$/);
	my ($symb,$uname,$udom) = &decode_queue_key($key);
	next if (!defined($classlist->{$uname.':'.$udom}));
	push(@users,"$uname:$udom");
    }
    return @users;
}

sub get_from_queue {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo=&find_mid_grade($queue,$symb,$cdom,$cnum);
    &Apache::lonxml::debug("found ".join(':',&decode_queue_key($todo)));
    if ($todo) { return $todo; }
    my $attempts=0;

    my $classlist=&get_limited_classlist();

    while (1) {
	if ($attempts > 2) {
	    # tried twice to get a queue entry, giving up
	    return (undef,'unable');
	}
	my $starttime=time;
	&Apache::lonnet::cput($queue,{"$symb\0timestamp"=>$starttime},
			      $cdom,$cnum);
	&Apache::lonxml::debug("$starttime");
	my $regexp="^$symb\0queue\0";
	#my $range= ($attempts < 1 ) ? '0-100' : '0-400';

	my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
	#make a pass looking for a user _not_ in my section
	if ($env{'request.course.sec'}) {
	    &Apache::lonxml::debug("sce");
	    $todo=&pick_from_queue_data($queue,$env{'request.course.sec'},
					\%queue,$cdom,$cnum,$classlist);
	    &Apache::lonxml::debug("sce $todo");
	}
	# no one _not_ in our section so look for any user that is
	# ready for grading
	if (!$todo) {
	    &Apache::lonxml::debug("no sce");
	    $todo=&pick_from_queue_data($queue,undef,\%queue,$cdom,$cnum,
					$classlist);
	    &Apache::lonxml::debug("no sce $todo");
	}
	# no user to grade 
	if (!$todo) { last; }
	&Apache::lonxml::debug("got $todo");
	# otherwise found someone so lets try to lock them
	# unless someone else already picked them
	if (!&lock_key($queue,$todo)) {
	    $attempts++;
	    next;
	}
	my (undef,$endtime)=
	    &Apache::lonnet::get($queue,["$symb\0timestamp"],
				 $cdom,$cnum);
	&Apache::lonxml::debug("emd  $endtime");
	# someone else already modified the queue, 
	# perhaps our picked user wass already fully graded between
	# when we picked him and when we locked his record? so lets
	# double check.
	if ($endtime != $starttime) {
	    my ($key,$value)=
		&Apache::lonnet::get($queue,["$todo"],
				     $cdom,$cnum);
	    &Apache::lonxml::debug("check  $key .. $value");
	    if ($key eq $todo && ref($value)) {
	    } else {
		&Apache::lonnet::del($queue,["$todo\0locked"],
				     $cdom,$cnum);
		&Apache::lonxml::debug("del");
		$attempts++;
		next;
	    }
	}
	&Apache::lonxml::debug("last $todo");
	last;
    }
    return $todo;
}

sub select_user {
    my ($symb,$cid)=&Apache::lonxml::whichuser();

    my @chosen_sections=
	&Apache::loncommon::get_env_multiple('form.chosensections');

    my $classlist = &get_limited_classlist(\@chosen_sections);
    
    my $result;
    if (!(grep(/^all$/,@chosen_sections))) {
	$result.='<p> Showing only sections <tt>'.join(', ',@chosen_sections).
	    '</tt>.</p> '."\n";
    }
    $result.=&Apache::loncommon::start_data_table();

    foreach my $student (sort {lc($classlist->{$a}[&Apache::loncoursedata::CL_FULLNAME()]) cmp lc($classlist->{$b}[&Apache::loncoursedata::CL_FULLNAME()]) } (keys(%$classlist))) {
	my ($uname,$udom) = split(/:/,$student);
	
	my $cnum=$env{'course.'.$cid.'.num'};
	my $cdom=$env{'course.'.$cid.'.domain'};
	my %status = &get_student_status($symb,$cdom,$cnum,$udom,$uname,
					 'Task');
	my $queue = 'none';
	my $cannot_grade;
	if ($status{'reviewqueue'} =~ /^(in_progress|enqueue)$/) {
	    $queue = 'reviewqueue';
	    if ($status{'reviewqueue'} eq 'in_progress') {
		$cannot_grade=1;
	    }
	} elsif ($status{'gradingqueue'} =~ /^(in_progress|enqueue)$/) {
	    $queue = 'gradingqueue';
	    if ($status{'gradingqueue'} eq 'in_progress') {
		$cannot_grade=1;
	    }
	}
	my $todo = 
	    &escape(&encode_queue_key($symb,$udom,$uname));
	if ($cannot_grade) {
	    $result.=&Apache::loncommon::start_data_table_row().
		'<td>&nbsp;</td><td>'.$classlist->{$student}[&Apache::loncoursedata::CL_FULLNAME()].
		'</td><td>';
	} else {
	    my $seclist;
	    foreach my $sec (@chosen_sections) {
		$seclist.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
	    }
	    $result.=&Apache::loncommon::start_data_table_row();
	    $result.=<<RESULT;
  <td>
    <form style="display: inline" method="post">
      <input type="hidden" name="gradingkey" value="$todo" />
      <input type="hidden" name="queue" value="$queue" />
      <input type="hidden" name="webgrade" value="no" />
      <input type="hidden" name="regrade" value="yes" />
      <input type="submit" name="submit" value="Regrade" />
      $seclist
    </form>
  <td>$classlist->{$student}[&Apache::loncoursedata::CL_FULLNAME()] <tt>($student)</tt></td>
  <td>
RESULT
        }
        if ($status{'status'} eq 'pass') {
	    $result .= '<font color="green">'.&mt('Passed').'</font>';
	} elsif ($status{'status'} eq 'fail') {
	    $result .= '<font color="red">'.&mt('Failed').'</font>';
	} elsif ($status{'status'} eq 'review') {
	    $result .= '<font color="blue">'.&mt('Under Review').'</font>';
	} elsif ($status{'status'} eq 'ungraded') {
	    $result .= &mt('Ungraded');
	} elsif ($status{'status'} ne '') {
	    $result .= '<font color="orange">'.&mt('Unknown Status').'</font>';
	} else {
	    $result.="&nbsp;";
	}
	if ($status{'version'}) {
	    $result .= ' '.&mt('Version').' '.$status{'version'};
	}
	if ($status{'grader'}) {
	    $result .= ' '.&mt('(Graded by [_1])',$status{'grader'}).' ';
	}
	$result.= '</td><td>';
	if ($status{'reviewqueue'} eq 'enqueued') {
	    $result .= &mt('Awaiting Review');
	} elsif ($status{'reviewqueue'} eq 'locked') {
	    $result .= &mt('Under Review');
	} elsif ($status{'reviewqueue'} eq 'in_progress') {
	    $result .= &mt('Still being worked on.');
	} elsif ($status{'gradingqueue'} eq 'enqueued') {
	    $result .= &mt('Awaiting Grading');
	} elsif ($status{'gradingqueue'} eq 'locked') {
	    $result .= &mt('Being Graded');
	} elsif ($status{'gradingqueue'} eq 'in_progress') {
	    $result .= &mt('Still being worked on.');
	} else {
	    $result.="&nbsp;";
	}
	$result.= '</td>'.&Apache::loncommon::end_data_table_row();
    }
    $result.=&Apache::loncommon::end_data_table();
    return $result;
}

sub get_student_status {
    my ($symb,$cdom,$cnum,$udom,$uname,$type)=@_;

    my %status;

    if ($type eq 'Task') {
	my %record = &Apache::lonnet::restore($symb,$env{'request.course.id'},
					  $udom,$uname);
	$status{'status'}=$record{'resource.0.status'};
	$status{'version'}=$record{'resource.0.version'};
	$status{'grader'}=$record{'resource.0.regrader'};
    }
    $status{'reviewqueue'}=
	&check_queue_for_key($cdom,$cnum,'reviewqueue',
			     &encode_queue_key($symb,$udom,$uname));
    $status{'gradingqueue'}=
	&check_queue_for_key($cdom,$cnum,'gradingqueue',
			     &encode_queue_key($symb,$udom,$uname));
    return %status;
}

sub start_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::startredirection();
    }
    return $result;
}

sub end_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::endredirection();
    }
    return $result;
}

sub get_dim_id {
    return $Apache::bridgetask::dimension[-1];
}

sub get_id {
    my ($parstack,$safeeval)=@_;
    my $id=&Apache::lonxml::get_param('id',$parstack,$safeeval);
    if (!$id) { $id=$Apache::lonxml::curdepth; }
    return $id;
}

sub start_Setup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    #undef(%dimension);
    my $dim = &get_id($parstack,$safeeval);
    push(@Apache::bridgetask::dimension,$dim);
    &Apache::lonxml::startredirection();
    return &internal_location($dim);
}

{
    my @allowed;
    sub enable_dimension_parsing {
	my ($id) = @_;
	push(@allowed,$id);
    }
    sub disable_dimension_parsing {
	pop(@allowed);
    }
    sub skip_dimension_parsing {
	my ($check) = @_;
	if (!@allowed) { return 0;}
	# if unspecified allow any id
	if ($allowed[-1] eq undef) { return 0;}

	return ($check ne $allowed[-1]);
    }
}

sub start_Question { return &start_Dimension(@_); }
sub start_Dimension {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $dim = &get_id($parstack,$safeeval);
    my $previous_dim;
    if (@Apache::bridgetask::dimension) {
	$previous_dim = $Apache::bridgetask::dimension[-1];
	push(@{$Apache::bridgetask::dimension{$previous_dim}{'contains'}},
	     $dim);
	if(&skip_dimension_parsing($dim)) {
	    $dimension{$previous_dim}{'criteria.'.$dim} =
		$token->[4]
		.&Apache::lonxml::get_all_text('/'.$tagstack->[-1],$parser,
					      $style)
		.'</'.$tagstack->[-1].'>';
	}
	$dimension{$previous_dim}{'criteria.'.$dim.'.type'}='dimension';
	$dimension{$previous_dim}{'criteria.'.$dim.'.mandatory'}=
	    &Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
	push(@{$dimension{$previous_dim}{'criterias'}},$dim);
	$dimension{$dim}{'nested'}=$previous_dim;
	&Apache::lonxml::debug("adding $dim as criteria to $previous_dim");
    } else {
	$Apache::bridgetask::top_dimensionlist{$dim}{'manadatory'}=
	    &Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
    }
    push(@Apache::bridgetask::dimension,$dim);
    &Apache::lonxml::startredirection();
    &enable_dimension_parsing($dim);
    return &internal_location($dim);
}

sub start_QuestionText {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $dim = &get_dim_id();
    my $text=&Apache::lonxml::get_all_text('/questiontext',$parser,$style);
    if ($target eq 'grade' || $target eq 'web' || $target eq 'webgrade') {
	$dimension{$dim}{'questiontext'}=$text;
    }
    return '';
}

sub end_QuestionText {
    return '';
}

sub get_instance {
    my ($dim)=@_;
    my $rand_alg=&Apache::lonnet::get_rand_alg();
    if (!$rand_alg || $rand_alg eq '32bit' || $rand_alg eq '64bit' ||
	$rand_alg eq '64bit2' || $rand_alg eq '64bit3' ||
	$rand_alg eq '64bit4' ) {
	&Apache::response::pushrandomnumber();
	my @order=&Math::Random::random_permutation(@{$dimension{$dim}{'instances'}});
	my $num=@order;
	my $version=&get_version();
	my $which=($version-1)%$num;
	return $order[$which];
    } else {
	my ($version,$previous) = &get_version();
	my $instance = 
	    $Apache::lonhomework::history{"resource.$version.0.$dim.instance"};
	if (defined($instance)) { return $instance; }

	&Apache::response::pushrandomnumber();
	if (ref($dimension{$dim}{'instances'}) eq 'ARRAY') {
	    my @instances = @{$dimension{$dim}{'instances'}};
	    # remove disabled instances
	    for (my $i=0; $i < $#instances; $i++) {
		if ($dimension{$dim}{$instances[$i].'.disabled'}) {
		    splice(@instances,$i,1);
		    $i--;
		}
	    }
	    @instances = &Math::Random::random_permutation(@instances);
	    $instance  = $instances[($version-1)%scalar(@instances)];
	    if ($version =~ /^\d$/) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.instance"} = 
		    $instance;
		$Apache::lonhomework::results{'INTERNAL_store'} = 1; 
	    }
	}
	&Apache::response::poprandomnumber();
	return $instance;
    }
}

sub get_criteria {
    my ($what,$version,$dim,$id) = @_;
    my $type = $dimension{$dim}{'criteria.'.$id.'.type'};
    my $prefix = ($type eq 'criteria') ? "$dim.$id"
	                               : "$id";
    my $entry = "resource.$version.0.$prefix.$what";
    if (exists($Apache::lonhomework::results{$entry})) {
	return $Apache::lonhomework::results{$entry};
    }
    return $Apache::lonhomework::history{$entry};
}

{
    my $last_link;
    sub link {
	my ($id) = @_;
	$id =~ s/\./_/g;
	return 'LC_GRADING_criteria_'.$id;
    }
    sub end_Question { return &end_Dimension(@_); }
    sub end_Dimension {
	my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
	my $result=&Apache::lonxml::endredirection();
	my $dim=&get_id($parstack,$safeeval);
	if (&skip_dimension_parsing($dim)) {
	    &disable_dimension_parsing($dim);
	    pop(@Apache::bridgetask::dimension);
	    return;
	}
	my $instance=&get_instance($dim);
	my $version=&get_version();
	if ($target eq 'web') {
	    $result .= &nested_parse(\$dimension{$dim}{'intro'},[@_]);
	    my @instances = $instance;
	    if (&Apache::response::showallfoils()) {
		@instances = @{$dimension{$dim}{'instances'}};
	    }
	    my $shown_question_text;
	    foreach my $instance (@instances) {
		$result .= &nested_parse(\$dimension{$dim}{$instance.'.text'},
					 [@_]);
		$result .= &nested_parse(\$dimension{$dim}{'questiontext'},
					 [@_],{'set_dim_id' => undef});
		my $task_status = 
		    $Apache::lonhomework::history{"resource.$version.0.status"};
		if ($task_status ne 'pass' && $task_status ne 'fail') {

		    foreach my $id (@{$dimension{$dim}{$instance.'.criterias'}},
				    @{$dimension{$dim}{'criterias'}}) {
			my $type = $dimension{$dim}{'criteria.'.$id.'.type'};
			&Apache::lonxml::debug("$id is $type");
			if ($type eq 'dimension') {
			    $result.=
				&nested_parse(\$dimension{$dim}{'criteria.'.$id},
					      [@_],{'set_dim_id' => $id});
			}
		    }
		} else {
		    my $dim_status=$Apache::lonhomework::history{"resource.$version.0.$dim.status"};
		    my $mandatory='Mandatory';
		    if ($Apache::bridgetask::dimensionmandatory{$dim} eq 'N') {
			$mandatory='Optional';
		    }
		    my $dim_info="<div class='LC_$dim_status LC_question_grade'>\n";
		    if ($dim_status eq 'pass') {
			$dim_info.='<h3>Question : you passed this '.$mandatory.' question</h3>';
		    }
		    if ($dim_status eq 'fail') {
			$dim_info.='<h3>Question : you did not pass this '.$mandatory.' question</h3>';
		    }
		    my $man_count=0;
		    my $man_passed=0;
		    my $opt_count=0;
		    my $opt_passed=0;
		    foreach my $id ( @{$dimension{$dim}{$instance.'.criterias'}},
				     @{$dimension{$dim}{'criterias'}} ) {
			my $status = &get_criteria('status',$version,$dim,$id);
			if ($dimension{$dim}{'criteria.'.$id.'.mandatory'} 
			    eq 'N') {
			    $opt_count++;
			    if ($status eq 'pass') { $opt_passed++; }
			} else {
			    $man_count++;
			    if ($status eq 'pass') { $man_passed++; }
			}
		    }
		    if ($man_passed eq $man_count) { $man_passed='all'; }

		    my $opt_req=$dimension{$dim}{$instance.'.optionalrequired'};
		    if ($opt_req !~ /\S/) {
			$opt_req=
			    &Apache::lonxml::get_param('OptionalRequired',
						       $parstack,$safeeval);
			if ($opt_req !~ /\S/) { $opt_req = 0; }
		    }
		    $dim_info.="\n<p>".&mt('You passed [_1] of the [_2] mandatory components and [_3] of the [_4] optional components, of which you were required to pass [_5].',$man_passed,$man_count,$opt_passed,$opt_count,$opt_req)."</p>\n</div>";

		    my $internal_location=&internal_location($dim);
		    $result=~s/\Q$internal_location\E/$dim_info/;

		    foreach my $id (@{$dimension{$dim}{$instance.'.criterias'}},
				    @{$dimension{$dim}{'criterias'}}) {
			my $type = $dimension{$dim}{'criteria.'.$id.'.type'};
			if ($type eq 'dimension') {
			    $result.=$dimension{$id}{'result'};
			    next;
			}
			my $status= &get_criteria('status', $version,$dim,$id);
			my $comment=&get_criteria('comment',$version,$dim,$id);
			my $mandatory=($dimension{$dim}{'criteria.'.$id.'.mandatory'} ne 'N');
			if ($mandatory) {
			    $mandatory='Mandatory';
			} else {
			    $mandatory='Optional';
			}
			if ($status eq 'fail') {
			} elsif ($status eq 'pass') {
			} else {
			    &Apache::lonxml::error("Student viewing a graded bridgetask was shown a status of $status");
			}
			my $status_display=$status;
			$status_display=~s/^([a-z])/uc($1)/e;
			$result.=
			    '<div class="LC_'.$status.' LC_criteria"><h4>'
			    .$mandatory.' Criteria</h4><p>';
			$result.=
			    &nested_parse(\$dimension{$dim}{'criteria.'.$id},
					  [@_],{'set_dim_id' => $id});
			$result.='</p><p class="LC_grade">'.$status_display.'</p>';
			if ($comment) {
			    $result.='<p class="LC_comment">'.
				&mt('Comment: [_1]',$comment).'</p>';
			}
			$result.='</div>';
		    }
		}
	    }
	} elsif ($target eq 'webgrade') {
	    # in case of any side effects that we need
	    &nested_parse(\$dimension{$dim}{'intro'},[@_]);
	    &nested_parse(\$dimension{$dim}{$instance.'.text'},[@_]);
	    $result.=
		&nested_parse(\$dimension{$dim}{'questiontext'},[@_],
			      {'set_dim_id'          => undef,
			       'delayed_dim_results' => 1});
	    foreach my $id (@{$dimension{$dim}{$instance.'.criterias'}},
			    @{$dimension{$dim}{'criterias'}} ) {
		my $type = $dimension{$dim}{'criteria.'.$id.'.type'};
		if ($type eq 'dimension') {
		    # dimensional 'criteria' don't get assigned grades
		    $result.=
			&nested_parse(\$dimension{$dim}{'criteria.'.$id},
				      [@_],{'set_dim_id' => $id});
		    next;
		}

		my $link=&link($id);
		my $status= &get_criteria('status',$version,$dim,$id);
		$result.='<div class="LC_GRADING_criteria" id="'.$link.'">'."\n".
		    '<div class="LC_GRADING_criteriatext" id="next_'.$last_link.'">'."\n";
		$result.=
		    &nested_parse(\$dimension{$dim}{'criteria.'.$id},[@_]);

		$result.='</div>'."\n".
		    '<div class="LC_GRADING_grade">'."\n".
		    '<label class="LC_GRADING_ungraded"><input type="radio" name="HWVAL_'.$link.'" value="ungraded" '.($status eq 'ungraded' || !$status ? 'checked="checked"':'').' />'.&mt('Ungraded').'</label>'."\n".
		    '<label class="LC_GRADING_fail"><input type="radio" name="HWVAL_'.$link.'" value="fail" '.($status eq 'fail' ? 'checked="checked"':'').' />'.&mt('Fail').'</label>'."\n".
		    '<label class="LC_GRADING_pass"><input type="radio" name="HWVAL_'.$link.'" value="pass" '.($status eq 'pass' ? 'checked="checked"':'').' />'.&mt('Pass').'</label>'."\n".
		    '<label class="LC_GRADING_review"><input type="radio" name="HWVAL_'.$link.'" value="review" '.($status eq 'review' ? 'checked="checked"':'').' />'.&mt('Review').'</label>'."\n".
		    '</div>'."\n".
		    '<label class="LC_GRADING_comment">'.&mt('Additional Comment for Student')."\n".
		    '<textarea class="LC_GRADING_comment_area" name="HWVAL_comment_'.$link.'">'.&HTML::Entities::encode(&get_criteria('comment',$version,$dim,$id),'<>"&').'</textarea>'."\n".
		    '</label>'."\n".
		    '<ul class="LC_GRADING_navbuttons">'."\n".
		    '<li><a href="#'.$last_link.'">Prev</a></li>'."\n".
		    '<li><a href="#next_'.$link.'">Next</a></li>'."\n".
		    '</ul>'."\n".
                    '</div>'."\n";
		$result.=&grading_history($version,$dim,$id);
		$last_link=$link;
	    }
	    if (&nest()) {
		&Apache::lonxml::debug(" for $dim stashing results into ".$dimension{$dim}{'nested'});
		$dimension{$dimension{$dim}{'nested'}}{'result'}.=$result;
		undef($result);
	    }
	} elsif ($target eq 'grade' && $env{'form.webgrade'}) {
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;

	    $result .= &nested_parse(\$dimension{$dim}{'intro'},[@_]);
	    $result .= &nested_parse(\$dimension{$dim}{$instance.'.text'},
				     [@_]);
	    $result .= &nested_parse(\$dimension{$dim}{'questiontext'},
				     [@_],{'set_dim_id' => undef});

	    foreach my $id (@{$dimension{$dim}{$instance.'.criterias'}},
			    @{$dimension{$dim}{'criterias'}}) {
		my $link=&link($id);

		my $type = $dimension{$dim}{'criteria.'.$id.'.type'};
		if ($type eq 'criteria') {
		    # dimensional 'criteria' don't get assigned grades
		    $Apache::lonhomework::results{"resource.$version.0.$dim.$id.status"}=$env{'form.HWVAL_'.$link};
		    $Apache::lonhomework::results{"resource.$version.0.$dim.$id.comment"}=$env{'form.HWVAL_comment_'.$link};
		} else {
		    $result .=
			&nested_parse(\$dimension{$dim}{'criteria.'.$id},
				      [@_],{'set_dim_id' => $id});
		}
		my $status= &get_criteria('status',$version,$dim,$id);

		my $mandatory=($dimension{$dim}{'criteria.'.$id.'.mandatory'} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'review') {
		    $review++;
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} else {
		    $ungraded++;
		}
	    }
	    # FIXME optional required can apply to only <instance> right now...
	    my $opt_req=$dimension{$dim}{$instance.'.optionalrequired'};
	    if ($opt_req !~ /\S/) {
		$opt_req=
		    &Apache::lonxml::get_param('OptionalRequired',
					       $parstack,$safeeval);
		if ($opt_req !~ /\S/) { $opt_req = 0; }
	    }
	    if ($optional_passed < $opt_req) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all instance ".join(':',@{$dimension{$dim}{$instance.'.criterias'}})." results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'review';
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'ungraded';
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'fail';
	    } else {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'pass';
	    }
	} else {
	    # any other targets no output
	    undef($result);
	}
	&disable_dimension_parsing();
	pop(@Apache::bridgetask::dimension);
	return $result;
    }

    sub end_Setup {
	my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
	my $result=&Apache::lonxml::endredirection();
	my $dim=&get_id($parstack,$safeeval);
	my $instance=&get_instance($dim);
	my $version=&get_version();
	if ($target eq 'web') {
	    @Apache::scripttag::parser_env = @_;
	    $result.=&Apache::scripttag::xmlparse($dimension{$dim}{'intro'});
	    my @instances = $instance;
	    if (&Apache::response::showallfoils()) {
		@instances = @{$dimension{$dim}{'instances'}};
	    }
	    foreach my $instance (@instances) {
		@Apache::scripttag::parser_env = @_;
		$result.=&Apache::scripttag::xmlparse($dimension{$dim}{$instance.'.text'});
		@Apache::scripttag::parser_env = @_;
		$result.=&Apache::scripttag::xmlparse($dimension{$dim}{'questiontext'});
	    }
	} elsif ($target eq 'webgrade' 
		 || $target eq 'grade' && $env{'form.webgrade'}) {
	    # in case of any side effects that we need
	    @Apache::scripttag::parser_env = @_;
	    &Apache::scripttag::xmlparse($dimension{$dim}{'intro'});
	    @Apache::scripttag::parser_env = @_;
	    &Apache::scripttag::xmlparse($dimension{$dim}{$instance.'.text'});
	    @Apache::scripttag::parser_env = @_;
	    &Apache::scripttag::xmlparse($dimension{$dim}{'questiontext'});
	} else {
	    # any other targets no output
	    undef($result);
	}
	pop(@Apache::bridgetask::dimension);
	return $result;
    }
}

sub grading_history {
    my ($version,$dim,$id) = @_;
    if (!&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
	return '';
    }
    my ($result,$grader);
    my $scope="resource.$version.0.$dim.$id";
    foreach my $t (1..$Apache::lonhomework::history{'version'}) {
	if (exists($Apache::lonhomework::history{$t.':resource.0.regrader'})) {
	    my ($gname,$gdom) = 
		split(':',$Apache::lonhomework::history{$t.':resource.0.regrader'});
	    my $fullname = &Apache::loncommon::plainname($gname,$gdom);
	    $grader = &Apache::loncommon::aboutmewrapper($fullname,
							 $gname,$gdom);
	}
	my $entry;
	if (exists($Apache::lonhomework::history{"$t:$scope.status"})) {
	    $entry.="<tt>".$Apache::lonhomework::history{"$t:$scope.status"}.'</tt>';
	}
	if (exists($Apache::lonhomework::history{"$t:$scope.comment"})) {
	    $entry.=' comment: "'.$Apache::lonhomework::history{"$t:$scope.comment"}.'"';
	}
	if ($entry) {
	    $result.= "<li>$grader : $entry </li>";
	}
    }
    if ($result) {
	return '<ul class="LC_GRADING_pastgrading">'.$result.'</ul>';
    }
    return '';
}

sub start_IntroParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    my $dim = &get_dim_id();
    if ($target eq 'grade' || $target eq 'web' || $target eq 'webgrade') {
	if ($tagstack->[-2] eq 'Dimension' || $tagstack->[-2] eq 'Question' ) {
	    $dimension{$dim}{'intro'}=
		&Apache::lonxml::get_all_text('/introparagraph',
					      $parser,$style);
       	} elsif ($tagstack->[-2] eq 'Task' && $target eq 'webgrade') {
	    &Apache::lonxml::startredirection();
	}
	
    }
    return $result;
}

sub end_IntroParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    if ($tagstack->[-2] eq 'Task' && $target eq 'webgrade') {
	my $result = &Apache::lonxml::endredirection();
    }
}

sub start_Instance {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $dim = &get_dim_id();
    my $id  = &get_id($parstack,$safeeval);
    push(@{$dimension{$dim}{'instances'}},$id);
    push(@{$Apache::bridgetask::instance{$dim}},$id);
    push(@Apache::bridgetask::instancelist,$id);
    $dimension{$dim}{$id.'.optionalrequired'}=
	&Apache::lonxml::get_param('OptionalRequired',$parstack,$safeeval);
    my $disabled = &Apache::lonxml::get_param('Disabled',$parstack,$safeeval);
    if (lc($disabled) eq 'yes') {
	$dimension{$dim}{$id.'.disabled'}='1';
    }
    return '';
}

sub end_Instance {
}

sub start_InstanceText {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $dim = &get_dim_id();
    my $instance_id=$Apache::bridgetask::instance{$dim}[-1];
    my $text=&Apache::lonxml::get_all_text('/instancetext',$parser,$style);
    if ($target eq 'grade' || $target eq 'web' || $target eq 'webgrade') {
	$dimension{$dim}{$instance_id.'.text'}=$text;
    }
    return '';
}

sub end_InstanceText {
    return '';
}

sub start_Criteria {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $criteria=&Apache::lonxml::get_all_text('/criteria',$parser,$style);
    if ($target eq 'web' || $target eq 'webgrade' || $target eq 'grade') {
	my $dim = &get_dim_id();
	my $id=&get_id($parstack,$safeeval);
	&Apache::lonxml::debug("Criteria $id with $dim");
	if (&Apache::londefdef::is_inside_of($tagstack,'Instance')) {
	    my $instance_id=$Apache::bridgetask::instance{$dim}[-1];
	    $dimension{$dim}{"criteria.$instance_id.$id"}=$criteria;
	    $dimension{$dim}{"criteria.$instance_id.$id.type"}='criteria';
	    $dimension{$dim}{"criteria.$instance_id.$id.mandatory"}=
		&Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
	    push(@{$dimension{$dim}{$instance_id.'.criterias'}},"$instance_id.$id");
	} else {
	    $dimension{$dim}{'criteria.'.$id}=$criteria;
	    $dimension{$dim}{'criteria.'.$id.'.type'}='criteria';
	    $dimension{$dim}{'criteria.'.$id.'.mandatory'}=
		&Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
	    push(@{$dimension{$dim}{'criterias'}},$id);
	}
    }
    return '';
}

sub end_Criteria {
}

sub proctor_validation_screen {
    my ($slot) = @_;
    my (undef,undef,$domain,$user) = &Apache::lonxml::whichuser();
    my $url=&Apache::lonnet::studentphoto($domain,$user,'jpg');
    my $name=&Apache::loncommon::plainname($user,$domain);
    
    my $msg;
    if ($env{'form.proctorpassword'}) {
	$msg='<p><font color="red">'.&mt("Failed to authenticate the proctor.")
	    .'</font></p>';
    }
    if (!$env{'form.proctordomain'}) { $env{'form.proctordomain'}=$domain; }
    my $result= (<<ENDCHECKOUT);
<h2>Proctor Validation</h2>
    <p>Your room's proctor needs to validate your access to this resource.</p>
    $msg
<form name="checkout" method="post" action="$env{'request.uri'}">
<input type="hidden" name="validate" value="yes" />
<input type="hidden" name="submitted" value="yes" />
<table>
  <tr><td>Proctor's Username:</td><td><input type="string" name="proctorname" value="$env{'form.proctorname'}" /></td></tr>
  <tr><td>Password:</td><td><input type="password" name="proctorpassword" value="" /></td></tr>
  <tr><td>Proctor's Domain:</td><td><input type="string" name="proctordomain" value="$env{'form.proctordomain'}" /></td></tr>
</table>
<input type="submit" name="checkoutbutton" value="Validate"  /><br />
<table border="1">
  <tr><td>
    <table>
      <tr><td colspan="2">Student who should be logged in is:</td></tr>
      <tr><td>Name:</td><td>$name</td></tr>
      <tr><td>Student ID:</td><td>$env{'environment.id'}</td></tr>
      <tr><td>Usename</td><td>$user:$domain</td></tr>
      <tr><td colspan="2"><img src="$url" /></td></tr>
    </table>
  </tr></td>
</table>
</form>
ENDCHECKOUT
    return $result;
}

1;
__END__
