# The LearningOnline Network
# "About Me" Personal Information
#
# $Id: lonaboutme.pm,v 1.42 2006/04/10 19:59:53 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonaboutme;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;
use Apache::lonrss();
use Apache::lonlocal;
use Apache::lonmsgdisplay();

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;
    my $target=$env{'form.grade_target'};
# ------------------------------------------------------------ Print the screen
    if ($target eq 'tex') {
	$r->print(&Apache::lonprintout::print_latex_header($env{'form.latex_type'}));
    }
    my (undef,undef,$cdom,$cnum)=split(/\//,$r->uri);
# Is this even a user?
    if (&Apache::lonnet::homeserver($cnum,$cdom) eq 'no_host') {
	&Apache::loncommon::simple_error_page($r,'No info',
					      'No user information available');
        return OK;
    }
# --------------------------------------------------------- The syllabus fields
    my %syllabusfields=&Apache::lonlocal::texthash(
       'aaa_contactinfo'   => 'Contact Information',
       'bbb_aboutme'       => 'About Me',
       'ccc_webreferences' => 'Web References');

# ------------------------------------------------------------ Get Query String
    &Apache::loncommon::get_unprocessed_cgi
                            ($ENV{'QUERY_STRING'},['forcestudent','register']);
# --------------------------------------------------------------- Force Student
    my $forcestudent='';
    if ($env{'form.forcestudent'}) { $forcestudent='student'; };
       
# --------------------------------------- There is such a user, get environment
    my %courseenv=&Apache::lonnet::dump('environment',$cdom,$cnum);
    if ($target ne 'tex') {
	my $start_page = 
	    &Apache::loncommon::start_page("Personal Information",undef,
					   {'function'       => $forcestudent,
					    'domain'         => $cdom,
					    'force_register' =>
						$env{'form.register'},});
	$r->print($start_page);
	$r->print('<h1>'.&Apache::loncommon::plainname($cnum,$cdom).'</h1>');
    } else {
	$r->print('\noindent{\large\textbf{'.&Apache::loncommon::plainname($cnum,$cdom).'}}\\\\\\\\');
    }
    if ($courseenv{'nickname'}) {
       $r->print(
         '<h2>&quot;'.$courseenv{'nickname'}.
         '&quot;</h2>');
    }
    if ($target ne 'tex') {
	$r->print('<h3>'.$Apache::lonnet::domaindescription{$cdom}.'</h3>'.
		  '<p>'.&Apache::loncommon::messagewrapper('Send me a message',$cnum,$cdom).'</p>'.&Apache::lonrss::advertisefeeds($cnum,$cdom));
    } else {
	$r->print('\textbf{'.$Apache::lonnet::domaindescription{$cdom}.'}\\\\');
    }
    my %syllabus=&Apache::lonnet::dump('aboutme',$cdom,$cnum);
    my $allowed=0;

# does this user have privileges to post, etc?

       my $privleged=$allowed=(($env{'user.name'} eq $cnum) && 
			       ($env{'user.domain'} eq $cdom));
       if ($forcestudent or $target eq 'tex') { $allowed=0; }
 
       if ($allowed) {
	   $r->print('<p><b>'.&mt('Privacy Note').':</b> '.
		     &mt('The information you submit can be viewed by anybody who is logged into LON-CAPA. Do not provide information that you are not ready to share publicly.').
		     '</p>'.
		     &Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'</p><p><a href="'.$r->uri.'?forcestudent=1">Show Public View</a>'.
		     &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
       } elsif ($privleged && $target ne 'tex') {
	   $r->print('<p><a href="'.$r->uri.'?forceedit=1"><font size="+1">'.
		     &mt('Edit').'</font></a></p>');
       }
      if (($env{'form.uploaddoc.filename'}) &&
          ($env{'form.storeupl'}) && ($allowed)) {
 	  if ($env{'form.uploaddoc.filename'}=~/\.(gif|jpg|png|jpeg)$/i) {
	      if ($syllabus{'uploaded.photourl'}) {
		  &Apache::lonnet::removeuploadedurl($syllabus{'uploaded.photourl'});
	      }
	      $syllabus{'uploaded.photourl'}=
                 &Apache::lonnet::userfileupload('uploaddoc',undef,'aboutme');
 	  }
          $syllabus{'uploaded.lastmodified'}=time;
          &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }
    if ($allowed && $env{'form.delupl'}) {
	if ($syllabus{'uploaded.photourl'}) {
	    &Apache::lonnet::removeuploadedurl($syllabus{'uploaded.photourl'});
	    delete($syllabus{'uploaded.photourl'});
	    &Apache::lonnet::del('aboutme',['uploaded.photourl'],$cdom,$cnum);
	}
    }
       if (($allowed) && ($env{'form.storesyl'})) {
	   foreach (keys %syllabusfields) {
               my $field=$env{'form.'.$_};
               $field=~s/\s+$//s;
               $field=&Apache::lonfeedback::clear_out_html($field,
                                                           $env{'user.adv'});
	       $syllabus{$_}=$field;
           }
           $syllabus{'uploaded.lastmodified'}=time;
           &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }

# ---------------------------------------------------------------- Get syllabus
    if (($syllabus{'uploaded.lastmodified'}) || ($allowed)) {
       my $lastmod=$syllabus{'uploaded.lastmodified'};
       $lastmod=($lastmod?&Apache::lonlocal::locallocaltime($lastmod):&mt('never'));
       $r->print(&mt('Last updated').': '.$lastmod);
       if ($syllabus{'uploaded.photourl'}) {
	   &Apache::lonnet::allowuploaded('/adm/aboutme',
					  $syllabus{'uploaded.photourl'});
	   my $image=
               qq{<img src="$syllabus{'uploaded.photourl'}" align="right" />};
	   if ($target eq 'tex') {
	       $image=&Apache::lonxml::xmlparse($r,'tex',$image);
	   }
	   $r->print($image);
       }
       if ($allowed) {
           $r->print(
	 '<form method="post">
               <input type="submit" name="delupl" value="Delete Photo" />
          </form>'.
	 '<form method="post" enctype="multipart/form-data">'.
         '<h3>'.&mt('Upload a Photo').'</h3>'.
         '<input type="file" name="uploaddoc" size="50">'.
         '<input type="submit" name="storeupl" value="Upload">'.
	 '</form><form method="post">');

       }
       foreach (sort keys %syllabusfields) {
          if (($syllabus{$_}) || ($allowed)) {
              my $message=$syllabus{$_};
	      &Apache::lonfeedback::newline_to_br(\$message);
              $message
             =~s/(http\:\/\/[^\s]+)/\<a href=\"$1\"\>\<tt\>$1\<\/tt\>\<\/a\>/g;
	      if ($allowed) {
		  $message=&Apache::lonspeller::markeduptext($message);
	      }
	      $message=&Apache::lontexconvert::msgtexconverted($message);
	      if ($target ne 'tex') {
		  $r->print('<h3>'.$syllabusfields{$_}.'</h3><blockquote>'.
                            $message.'</blockquote>');
	      } else {
		     $r->print('\\\\\textbf{'.$syllabusfields{$_}.'}\\\\'.
			       &Apache::lonxml::xmlparse($r,'tex',$message).'\\\\');
	      }
              if ($allowed) {
                 $r->print('<br /><textarea cols="80" rows="6" name="'.$_.'">'.
			   $syllabus{$_}.
           '</textarea><input type="submit" name="storesyl" value="'.
			   &mt('Store').'" />');
	      }
	  }
       }
       if ($allowed) {
	   $r->print('</form>');
       }
       if ($target ne 'tex') {$r->print('</p>');} else {$r->print('\\\\');}
    } else {
       $r->print('<p>'.&mt('No personal information provided').'.</p>');
    }
    if ($env{'request.course.id'}) {
	if (&Apache::lonnet::allowed('srm',$env{'request.course.id'})) {
	    if ($target ne 'tex') {
		$r->print('<a name="coursecomment" />');
		$r->print('<hr /><h3>'.
                          &mt('User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course').'</h3>'.
                          &mt('Shared by course faculty and staff').
                          &Apache::loncommon::help_open_topic("Course_Face_To_Face_Records,Course_Critical_Message").
'<br />');
                &Apache::lonmsgdisplay::disfacetoface($r,$cnum,$cdom);
                $r->print('<hr />');
                if (&Apache::lonnet::allowed('vsa',
                                             $env{'request.course.id'}) ||
                    &Apache::lonnet::allowed('vsa',
                                             $env{'request.course.id'}.'/'.
                                             $env{'request.course.sec'})) {
                    $r->print(&Apache::loncommon::track_student_link
                              ('View recent activity by this student',
                               $cnum,$cdom).('&nbsp;'x2));
                }
                $r->print(&Apache::loncommon::noteswrapper('Add Records',$cnum,$cdom));
	    } else {
		$r->print('\\\\\textbf{'.&mt('User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course').'}\\\\'.&mt('Shared by course faculty and staff').'\\\\\\\\');
		&Apache::lonmsgdisplay::disfacetoface($r,$cnum,$cdom);
	    }
        }
    }
    if ($target ne 'tex') {
	$r->print(&Apache::loncommon::end_page());
    } else {
	$r->print('\end{document}');
    }
    return OK;
} 

1;
__END__
