# The LearningOnline Network
# Pick a student from the classlist
#
# $Id: lonpickstudent.pm,v 1.19 2006/06/02 13:58:31 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonpickstudent;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::loncoursedata;
use Apache::lonnet;
use Apache::lonlocal;
use Apache::longroup;

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ------------------------------------------------------------ Print the screen
    $r->print(&Apache::loncommon::start_page("Selecting a User",undef,
					     {'no_nav_bar' => 1}));


    &Apache::loncommon::get_unprocessed_cgi
        ($ENV{'QUERY_STRING'},['filter','form','unameelement','udomelement',
			       'roles']);
# Allowed?
    my $allowed;
    my $scope = $env{'request.course.id'};
    if (!($allowed = &Apache::lonnet::allowed('srm',$scope))) {
	$scope .= '/'.$env{'request.course.sec'};
	$allowed = &Apache::lonnet::allowed('srm',$scope);
	if ($allowed) { $allowed = 'section';	}
    }

    unless (($env{'form.roles'}) ||
            (($env{'request.course.id'}) && ($allowed))) {
	$r->print(&mt('No context.').
		  &Apache::loncommon::end_page());
        return OK;
    }

# See if filter present

    my $filter=$env{'form.filter'};
    $filter=~s/\W//g;
    unless ($filter) { $filter='.'; }

    my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
    my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
    my $classlist=&Apache::loncoursedata::get_classlist();
    my %grouplist=&Apache::lonnet::get_group_membership($cdom,$cnum);
    my $now = time;
    my %allgroups = &Apache::longroup::coursegroups($cdom,$cnum);

# --------------------------------------- There is such a user, get environment

    $r->print(<<ENDSCRIPT);
<script>
function gochoose(uname,udom) {
    opener.document.$env{'form.form'}.$env{'form.unameelement'}.value=uname;
    var slct=opener.document.$env{'form.form'}.$env{'form.udomelement'};
    var i;
    for (i=0;i<slct.length;i++) {
        if (slct.options[i].value==udom) { slct.selectedIndex=i; }
    }
    self.close();
}
</script>
ENDSCRIPT
    
  $r->print('<form>');
  if ($allowed && (!$env{'form.roles'})) {
# -------------------------------------------------------- Get course personnel
    $r->print('<h3>'.$env{'course.'.$env{'request.course.id'}.'.description'}.
              '</h3>');
    my %coursepersonnel=
       &Apache::lonnet::get_course_adv_roles();
    $r->print('<table border="2">');
    foreach my $role (sort keys %coursepersonnel) {
       foreach (split(/\,/,$coursepersonnel{$role})) {
	   my ($puname,$pudom)=split(/\:/,$_);
           $role = &Apache::lonnet::plaintext($role,
                                            &Apache::loncommon::course_type());
	   $r->print('<tr><td>'.
             '<input type="button" value="Select" onClick="gochoose('.
             "'".$puname."','".$pudom."')".'" /></td><td>'.$role.'</td><td>'.
		     &Apache::loncommon::aboutmewrapper(
                          &Apache::loncommon::plainname($puname,
                          $pudom),$puname,$pudom).'</td></tr>');
	}
    }
    $r->print('</table><p>&nbsp;');
    if ($filter ne '.') {
       $r->print('<br/ >'.&mt('Name starting with').' "'.$filter.'"<br />');
    }
    $r->print('</p><p><table><tr><th>&nbsp;</th><th>username</th><th>domain</th><th>Name</th><th>ID</th><th>section</th><th>active group(s)</th></tr>');
  # ------------------------------------------------------------------ Students
    foreach (sort keys %$classlist) {
        # the following undefs are for 'domain', and 'username' respectively.
	my (undef,undef,$end,$start,$id,$section,$fullname,$status)=
            @{$classlist->{$_}};
	if ($allowed eq 'section' && $section ne $env{'request.course.sec'}) {
	    next;
	}
        if ($_=~/^(\w+)\:(\w+)$/) {
	    my ($uname,$udom)=($1,$2);
	    if (($uname=~/^$filter/) || 
                ($fullname=~/^$filter/i)) {
               my $grouplist = '';
               foreach my $group (sort(keys(%allgroups))) {
                   if (exists($grouplist{$group.':'.$uname.':'.$udom})) {
                       my ($end,$start) = split(/:/,$grouplist{$group.':'.$uname.':'.$udom});
                       if (($end!=0) && ($end<$now)) { next; }
                       if (($start!=0) && ($start>$now)) {next; }
                       $grouplist .= " $group,"; 
                   }
               }
               $grouplist =~ s/,$//;
	       $r->print('<tr><td>'.
                      '<input type="button" value="'.&mt('Select').'" onClick="gochoose('.
                      "'".$uname."','".$udom."')".'" /></td>'.
                         '<td><tt>'.$uname.'</tt></td><td> <tt>'.$udom.
                         '</tt></td><td>'.
                         &Apache::loncommon::aboutmewrapper(
                         $fullname,
                         $uname,$udom).'</td><td>'.$id.'</td><td>'.$section.
                                       '</td><td>'.$grouplist.'</td></tr>');
	   }
        }
    }

    $r->print('</table></p>');
  } else {
    $r->print('<h3>'.&mt('Users with Roles Assigned by').' '.
	      &Apache::loncommon::plainname($env{'user.name'},
					    $env{'user.domain'}).'</h3>');
    if ($filter ne '.') {
       $r->print('<br/ >'.&mt('Name starting with').' "'.$filter.'"<br />');
    }
    $r->print('<p><table>');
    my %users=&Apache::lonnet::get_my_roles();
    foreach (sort keys %users) {
        if ($_=~/^(\w+)\:(\w+)\:(\w+)$/) {
	    my ($uname,$udom,$urole)=($1,$2,$3);
	    my $fullname=&Apache::loncommon::plainname($uname,$udom);
	    if (($uname=~/^$filter/) || 
                ($fullname=~/^$filter/i)) {
	       $r->print('<tr><td>'.
                      '<input type="button" value="Select" onClick="gochoose('.
                      "'".$uname."','".$udom."')".'" /></td>'.
                         '<td><tt>'.$uname.'</tt></td><td><tt>'.$udom.
                         '</tt></td><td>'.
                         &Apache::loncommon::aboutmewrapper(
                         $fullname,
                         $uname,$udom).'</td><td><td>'.
			 &Apache::lonnet::plaintext($urole).
                                       '</td></tr>');
	   }
        }
    }
    $r->print('</table></p>');
  }
  $r->print('</form>'.&Apache::loncommon::end_page());
  return OK;
} 

1;
__END__
